/*****************************************************************************
 *                                                                           *
 * PIC-232 - Comunicazione seriale per gestione memoria EEPROM 877 System    *
 * Copyright (C) 2009  by Walter Lain <kcswalter@member.fsf.org>             *
 *                                                                           *
 * This program is free software; you can redistribute it and/or modify      *
 * it under the terms of the GNU General Public License as published by      *
 * the Free Software Foundation; either version 3 of the License, or         *
 * (at your option) any later version.                                       *
 *                                                                           *
 * This program is distributed in the hope that it will be useful,           *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 * GNU General Public License for more details.                              *
 *                                                                           *
 * You should have received a copy of the GNU General Public License         *
 * along with this program; if not, write to the Free Software               *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
 *                                                                           *
 *****************************************************************************/

#include "params.h"			//definizioni programma


/********************************
 *          e2shownum()         *
 ********************************
 Mostra il contenuto della EEPROM in versione NUM

Variabili esterne:
dati=struttura SYSDATA (vedere params.h)
strings=struttura SYSTEXT (vedere params.h)

Variabili interne:
c=contatore colonna
ins=lettura input utente
page=pagina visualizzata
pages=totale pagine da visualizzare
r=contatore riga
rowcount=righe disponibili per la visualizzazione
rowlennum=lunghezza riga (in celle visualizzabili)
rows,cols=geometria schermo
*/
void e2shownum(SYSDATA *dati,SYSTEXT *strings)
{
   int r=0,page=0,pages,c,ins,rowlennum,rowcount,rows,cols;
   while(true)
   {
      clear();
      header(dati,strings);
      getmaxyx(stdscr, rows, cols);	//recupera la dimensione dello schermo
      if(rows<rowmin)
	rows=rowmin;
      if(cols<colmin)
	cols=colmin;
      rowlennum=(int)(cols/6)-1;
      rowcount=rows-8;
      for(c=0;(r<ceil(((float)(dati->eesize))/rowlennum))&&(r<rowcount);r++) //visualizza la matrice EEPROM
      {
	if(((r+(page*rowcount))*rowlennum)<16)
	  mvprintw(r+5,0,"00");
	else if((r+(page*rowcount))<ceil(256.0/rowlennum))
	  mvprintw(r+5,0,"0");
	else
	  move(r+5,0);
	printw("%x:",((r+(page*rowcount))*rowlennum));
	mvaddch(r+5,5,ACS_VLINE);
	for(c=0;(c<rowlennum)&&(((r+(page*rowcount))*rowlennum)+c<(dati->eesize));c++)
	{
	  if((dati->ee[((r+(page*rowcount))*rowlennum)+c])<10)
	    mvprintw(r+5,c+9+(5*c),"%i",dati->ee[((r+(page*rowcount))*rowlennum)+c]);
	  else if((dati->ee[((r+(page*rowcount))*rowlennum)+c])<100)
	    mvprintw(r+5,c+8+(5*c),"%i",dati->ee[((r+(page*rowcount))*rowlennum)+c]);
	  else
	    mvprintw(r+5,c+7+(5*c),"%i",dati->ee[((r+(page*rowcount))*rowlennum)+c]);
	  mvaddch(r+5,c+11+(5*c),ACS_VLINE);
	}
	if(((r+(page*rowcount))*rowlennum)+c>=(dati->eesize))
	{
	  break;
	}
      }
      pages=ceil((float)(dati->eesize)/(rowcount*rowlennum));
      mvprintw(rows-2,1,"%s",strings->pgupdown001);	//Press END to exit, PGUP and PGDOWN to scroll.
      printw(" %s %i %s %i",strings->str002,page+1,strings->str003,pages);	//Page * of *
      curs_set(0);
      refresh();
      ins=getch();
      switch(ins)
      {
	case KEY_PPAGE:			//pag su
	{
	  r=0;				//prima riga
	  if(page>0)
	    page--;			//pagina precedente
	  else
	    page=ceil((float)(dati->eesize)/(rowcount*rowlennum))-1;	//ultima pagina
	  break;
	}
	case KEY_NPAGE:			//pag giu
	{
	  r=0;				//prima riga
	  page++;			//pagina successiva
	  if(page>=ceil((float)(dati->eesize)/(rowcount*rowlennum)))
	    page=0;			//prima pagina
	  break;
	}
	case KEY_END:			//end
	{
	  return;
	}
	default:
	{
	  break;
	}
      }
   }
   return;
}


/********************************
 *         e2showchar()         *
 ********************************
 Mostra il contenuto della EEPROM in versione carattere
 
Variabili esterne:
dati=struttura SYSDATA (vedere params.h)
strings=struttura SYSTEXT (vedere params.h)

Variabili interne:
c=contatore colonna
ins=lettura input utente
page=pagina visualizzata
pages=totale pagine da visualizzare
r=contatore riga
rowcount=righe disponibili per la visualizzazione
rowlenchar=lunghezza riga (in celle visualizzabili)
rows,cols=geometria schermo
*/
void e2showchar(SYSDATA *dati,SYSTEXT *strings)
{
   int c,ins,r=0,page=0,pages,rowcount,rowlenchar,rows,cols;
   while(true)
   {
      clear();
      header(dati,strings);
      getmaxyx(stdscr, rows, cols);	//recupera la dimensione dello schermo
      if(rows<rowmin)
	rows=rowmin;
      if(cols<colmin)
	cols=colmin;
      rowcount=rows-8;
      rowlenchar=cols-11;
      move(4,0);
      for(c=0;(r<ceil(((float)(dati->eesize))/rowlenchar))&&(r<rowcount);r++) //visualizza la matrice EEPROM
      {
	if((r+page==0)||(((r+(page*rowcount))*rowlenchar)<16))
	  mvprintw(r+5,0,"00");
	else if((r+(page*rowcount))<ceil(256.0/rowlenchar))
	  mvprintw(r+5,0,"0");
	else
	  move(r+5,0);
	printw("%x: >>",((r+(page*rowcount))*rowlenchar));
	for(c=0;(c<rowlenchar)&&(((r+(page*rowcount))*rowlenchar)+c<(dati->eesize));c++)
	{
	  if(((dati->ee[((r+(page*rowcount))*rowlenchar)+c])>31)&&((dati->ee[((r+(page*rowcount))*rowlenchar)+c])<127))
	    mvprintw(r+5,c+8,"%c",dati->ee[((r+(page*rowcount))*rowlenchar)+c]);	//evita di riprodurre caratteri schifosi
	  else
	    mvaddch(r+5,c+8,ACS_CKBOARD);
	}
	if(((r+(page*rowcount))*rowlenchar)+c>=(dati->eesize))
	{
	  mvprintw(r+5,c+9,"<<");
	  break;
	}
	else
	  mvprintw(r+5,(cols-2),"<<");
      }
      pages=ceil((float)(dati->eesize)/(rowcount*rowlenchar));	//calcola pagine totali
      mvprintw(rows-2,1,"%s",strings->pgupdown001);	//Press END to exit, PGUP and PGDOWN to scroll.
      printw(" %s %i %s %i",strings->str002,page+1,strings->str003,pages);	//Page * of *
      curs_set(0);
      refresh();
      ins=getch();
      switch(ins)
      {
	case KEY_PPAGE:			//pag su
	{
	  r=0;				//prima riga
	  if(page>0)
	    page--;			//pagina precedente
	  else
	    page=ceil((float)(dati->eesize)/(rowcount*rowlenchar))-1;	//ultima pagina
	  break;
	}
	case KEY_NPAGE:			//pag giu
	{
	  r=0;				//prima riga
	  page++;			//pagina successiva
	  if(page>=ceil((float)(dati->eesize)/(rowcount*rowlenchar)))
	    page=0;			//prima pagina
	  break;
	}
	case KEY_END:			//end
	{
	  return;
	}
	default:
	{
	  break;
	}
      }
   }
   return;
}


/********************************
 *          e2modnum()          *
 ********************************
 Modifica il contenuto della EEPROM in versione NUM
 
Variabili esterne:
dati=struttura SYSDATA (vedere params.h)
strings=struttura SYSTEXT (vedere params.h)

Variabili interne:
c=contatore colonna
col=posizione cursore cella
ins=lettura input utente
page=pagina visualizzata
pages=totale pagine da visualizzare
r=contatore riga
row=posizione cursore riga
rowcount=righe disponibili per la visualizzazione
rowlennum=lunghezza riga (in celle visualizzabili)
rows,cols=geometria schermo
total=accumulatore per conversione da char(1,2 o 3 cifre) a int
*/
void e2modnum(SYSDATA *dati,SYSTEXT *strings)
{
   int c,ins,r=0,row=0,col=0,page=0,pages,rowlennum,rowcount,total,rows,cols;
   while(true)
   {
      clear();
      header(dati,strings);
      getmaxyx(stdscr, rows, cols);	//recupera la dimensione dello schermo
      if(rows<rowmin)
	rows=rowmin;
      if(cols<colmin)
	cols=colmin;
      rowlennum=(int)(cols/6)-1;
      rowcount=rows-8;
      move(4,0);
      for(r=0,c=0;(r<ceil(((float)(dati->eesize))/rowlennum))&&(r<rowcount);r++) //visualizza la matrice EEPROM
      {
	if(((r+(page*rowcount))*rowlennum)<16)
	  mvprintw(r+5,0,"00");
	else if((r+(page*rowcount))<ceil(256.0/rowlennum))
	  mvprintw(r+5,0,"0");
	else
	  move(r+5,0);
	printw("%x:",((r+(page*rowcount))*rowlennum));
	mvaddch(r+5,5,ACS_VLINE);
	for(c=0;(c<rowlennum)&&(((r+(page*rowcount))*rowlennum)+c<(dati->eesize));c++)
	{
	  if(r==row&&c==col)
	    attron(A_REVERSE);		//evidenzia la cella
	  if((dati->ee[((r+(page*rowcount))*rowlennum)+c])<10)
	    mvprintw(r+5,c+7+(5*c),"  %u",dati->ee[((r+(page*rowcount))*rowlennum)+c]);
	  else if((dati->ee[((r+(page*rowcount))*rowlennum)+c])<100)
	    mvprintw(r+5,c+7+(5*c)," %u",dati->ee[((r+(page*rowcount))*rowlennum)+c]);
	  else
	    mvprintw(r+5,c+7+(5*c),"%u",dati->ee[((r+(page*rowcount))*rowlennum)+c]);
	  attroff(A_REVERSE); 		//fine evidenziatura
	  mvaddch(r+5,c+11+(5*c),ACS_VLINE);
	}
	if(((r+(page*rowcount))*rowlennum)+c>=(dati->eesize))
	{
	  break;
	}
      }
      pages=ceil((float)(dati->eesize)/(rowcount*rowlennum));	//calcola totale pagine
//		    --------------------------------------------------------------------------------
      mvprintw(rows-2,1,"%s",strings->str001);	//Selected cell
      if(((row+(rowcount*page))*rowlennum)+col<16)
      {
	printw(": 00%x - ",(((row+(rowcount*page))*rowlennum)+col));
	printw("%s %i %s %i - %s",strings->str002,page+1,strings->str003,pages,strings->str004);//Page * of * - Arrow keys to move, END to terminate...
      }
      else if(((row+(rowcount*page))*rowlennum)+col<256)
      {
	printw(": 0%x - ",(((row+(rowcount*page))*rowlennum)+col));
	printw("%s %i %s %i - %s",strings->str002,page+1,strings->str003,pages,strings->str004);//Page * of * - Arrow keys to move, END to terminate...
      }
      else
      {
	printw(": %x - ",(((row+(rowcount*page))*rowlennum)+col));
	printw("%s %i %s %i - %s",strings->str002,page+1,strings->str003,pages,strings->str004);//Page * of * - Arrow keys to move, END to terminate...
      }
//      printw("\n ROW: %i  COL: %i  PAGE: %i  PAGES: %i  RCOUNT*PAGE: %i",row,col,page,pages,rowcount*page);
      mvprintw(rows-1,2,"%s",strings->str005);	//CELL value (write 3 digits):
      move(rows-1,strlen(strings->str005)+3);
      curs_set(1);
      refresh();
      ins=getch();
      if(ins==KEY_UP)			//freccia su
      {
	if(row>0)
	{
	  row--;			//riga precedente
	}
	else
	{
	  if(page>0)
	    page--;			//pagina precedente
	  else
	  {
	    page=(pages-1);		//ultima pagina
	  }
	  row=ceil(((float)(dati->eesize)-(rowlennum*rowcount*page))/rowlennum)-1;
	  while(((rowlennum*rowcount*page)+(row*rowlennum)>(dati->eesize))||(row>rowcount-1))
	    row--;			//ultima riga della pagina
	}
	while(((((rowcount*page)+row)*rowlennum)+col>((dati->eesize)-1))&&(col>0))	//se la riga non e' a lunghezza piena, potrebbe
	  col--;								//trovarsi fuori campo. Corregge
      }
      else if(ins==KEY_DOWN)		//freccia giu
      {
	if(row<(rowcount-1)&&(row<ceil(((float)(dati->eesize)-(rowlennum*rowcount*page))/rowlennum)-1))
	  row++;			//riga successiva
	else
	{
	  row=0;			//prima riga
	  if(page<(pages-1))
	    page++;			//pagina successiva
	  else
	    page=0;			//prima pagina
	}
	while(((((rowcount*page)+row)*rowlennum)+col>((dati->eesize)-1))&&(col>0))	//se la riga non e' a lunghezza piena, potrebbe
	  col--;								//trovarsi fuori campo. Corregge
      }
      else if(ins==KEY_RIGHT)		//freccia dx
      {
	col++;				//colonna successiva
	if(col>(rowlennum-1))
	{
	  col=0;			//prima colonna
	  if(row<rowcount-1)
	  {
	    row++;			//riga successiva
	  }
	  else
	  {
	    if((rowcount*page)+row<(rowcount-1))
	      row++;			//riga successiva
	    else
	    {
	      row=0;			//prima riga
	      if(page<(pages-1))
		page++;			//pagina successiva
	      else
		page=0;			//prima pagina
	    }
	  }
	}
	if((((rowcount*page)+row)*rowlennum)+col>((dati->eesize)-1))
	{
	  row=0;
	  col=0;
	  page=0;
	}
      }
      else if(ins==KEY_LEFT)		//freccia sx
      {
	col--;				//colonna precedente
	if(col<0)
	{
	  col=rowlennum-1;		//ultima colonna
	  if(row>0)
	  {
	    row--;			//riga precedente
	  }
	  else
	  {
	    if(page>0)
	      page--;			//pagina precedente
	    else
	    {
	      page=(pages-1);		//ultima pagina
	    }
	    row=ceil(((float)(dati->eesize)-(rowlennum*rowcount*page))/rowlennum)-1;
	    while(((rowlennum*rowcount*page)+(row*rowlennum)>(dati->eesize))||(row>rowcount-1))
	      row--;			//ultima riga della pagina
	  }
	  while(((((rowcount*page)+row)*rowlennum)+col>((dati->eesize)-1))&&(col>0))	//se la riga non e' a lunghezza piena, potrebbe
	    col--;								//trovarsi fuori campo. Corregge
	}
      }
      else if(ins==KEY_END)		//END
      {
	curs_set(0);
	return;
      }
      else if(ins>47&&ins<58)		//gestisce solo i numeri
      {
	total=ins-48;
	mvprintw(rows-1,strlen(strings->str005)+3,"%i",ins-48);
	refresh();
	ins=getch();
	while(ins<48||ins>57)		//loop finche' non sono inserite tutte e tre le cifre
	  ins=getch();
	total=(total*10)+(ins-48);
	mvprintw(rows-1,strlen(strings->str005)+4,"%i",ins-48);
	refresh();
	ins=getch();
	while(ins<48||ins>57)		//loop finche' non sono inserite tutte e tre le cifre
	  ins=getch();
	total=(total*10)+(ins-48);
	mvprintw(rows-1,strlen(strings->str005)+5,"%i",ins-48);
	refresh();
	if(total>255)			//il valore non puo' superare 255
	  total=255;
	dati->ee[((row*rowlennum)+col)]=total;//modifica cella
	dati->eemod[((row*rowlennum)+col)]=true;	//setta cella modificata
	if(col<rowlennum-1)		//sposta il cursore in avanti
	  col++;			//colonna successiva
	else if(col==rowlennum-1)
	{
	  col=0;			//prima colonna
	  if(row<((dati->eesize)/rowlennum)-1)
	    row++;			//riga successiva
	  else
	    row=0;			//prima riga
	}
      }
   }
   return;
}


/********************************
 *          e2modchar()         *
 ********************************
 Modifica il contenuto della EEPROM in versione CHAR
 
Variabili esterne:
dati=struttura SYSDATA (vedere params.h)
strings=struttura SYSTEXT (vedere params.h)

Variabili interne:
c=contatore colonna
col=posizione cursore cella
ins=lettura input utente
page=pagina visualizzata
pages=totale pagine da visualizzare
r=contatore riga
row=posizione cursore riga
rowcount=righe disponibili per la visualizzazione
rowlenchar=lunghezza riga (in celle visualizzabili)
rows,cols=geometria schermo
*/
void e2modchar(SYSDATA *dati,SYSTEXT *strings)
{
   int c,ins,r=0,row=0,col=0,page=0,pages,rowcount,rowlenchar,rows,cols;
   while(true)
   {
      clear();
      header(dati,strings);
      getmaxyx(stdscr, rows, cols);	//recupera la dimensione dello schermo
      if(rows<rowmin)
	rows=rowmin;
      if(cols<colmin)
	cols=colmin;
      rowcount=rows-8;			//calcola righe disponibili
      rowlenchar=cols-11;		//calcola celle per riga
      for(r=0,c=0;(r<ceil(((float)(dati->eesize))/rowlenchar))&&(r<rowcount);r++) //visualizza la matrice EEPROM
      {
	  if(((r+(page*rowcount))*rowlenchar)<16)
	    mvprintw(r+5,0,"00");
	  else if((r+(page*rowcount))<ceil(256.0/rowlenchar))
	    mvprintw(r+5,0,"0");
	  else
	    move(r+5,0);
	  printw("%x: >>",((r+(page*rowcount))*rowlenchar));
	  
	  for(c=0;(c<rowlenchar)&&(((r+(page*rowcount))*rowlenchar)+c<(dati->eesize));c++)
	  {
	    if(r==row&&c==col)
	      attron(A_REVERSE|A_BLINK);//evidenzia la cella
	    if(((dati->ee[((r+(page*rowcount))*rowlenchar)+c])>31)&&((dati->ee[((r+(page*rowcount))*rowlenchar)+c])<127))
	      mvprintw(r+5,c+8,"%c",dati->ee[((r+(page*rowcount))*rowlenchar)+c]);	//evita di riprodurre caratteri schifosi
	    else
	      mvaddch(r+5,c+8,ACS_CKBOARD);
	    attroff(A_REVERSE|A_BLINK);	//fine evidenziatura
	  }
	  if(((r+(page*rowcount))*rowlenchar)+c>=(dati->eesize))
	  {
	    mvprintw(r+5,c+9,"<<");
	    break;
	  }
	  else
	    mvprintw(r+5,(cols-2),"<<");
      }
      pages=ceil((float)(dati->eesize)/(rowcount*rowlenchar));	//calcola totale pagine
//		      --------------------------------------------------------------------------------
      mvprintw(rows-2,1,"%s",strings->str001);	//Selected cell
      if(((row+(rowcount*page))*rowlenchar)+col<16)
      {
	printw(": 00%x - ",(((row+(rowcount*page))*rowlenchar)+col));
	printw("%s %i %s %i - %s",strings->str002,page+1,strings->str003,pages,strings->str004);//Page * of * - Arrow keys to move, END to terminate...
      }
      else if(((row+(rowcount*page))*rowlenchar)+col<256)
      {
	printw(": 0%x - ",(((row+(rowcount*page))*rowlenchar)+col));
	printw("%s %i %s %i - %s",strings->str002,page+1,strings->str003,pages,strings->str004);//Page * of * - Arrow keys to move, END to terminate...
      }
      else
      {
	printw(": %x - ",(((row+(rowcount*page))*rowlenchar)+col));
	printw("%s %i %s %i - %s",strings->str002,page+1,strings->str003,pages,strings->str004);//Page * of * - Arrow keys to move, END to terminate...
      }
//      mvprintw(23,1,"ROW: %i  COL: %i  PAGE: %i  PAGES: %i  RCOUNT*PAGE: %i",row,col,page,pages,rowcount*page);
      curs_set(0);
      move(row+5,col+8);		//posiziona il cursore sulla cella selezionata
      refresh();
      ins=getch();
      if(ins==KEY_UP)			//freccia su
      {
	if(row>0)
	{
	  row--;			//riga precedente
	}
	else
	{
	  if(page>0)
	    page--;			//pagina precedente
	  else
	  {
	    page=(pages-1);		//ultima pagina
	  }
	  row=ceil(((float)(dati->eesize)-(rowlenchar*rowcount*page))/rowlenchar)-1;
	  while(((rowlenchar*rowcount*page)+(row*rowlenchar)>(dati->eesize))||(row>rowcount-1))
	    row--;			//ultima riga della pagina
	}
	while(((((rowcount*page)+row)*rowlenchar)+col>((dati->eesize)-1))&&(col>0))	//se la riga non e' a lunghezza piena, potrebbe
	  col--;								//trovarsi fuori campo. Corregge
      }
      else if(ins==KEY_DOWN)		//freccia giu
      {
	if(row<(rowcount-1)&&(row<ceil(((float)(dati->eesize)-(rowlenchar*rowcount*page))/rowlenchar)-1))
	  row++;			//riga successiva
	else
	{
	  row=0;			//prima riga
	  if(page<(pages-1))
	    page++;			//pagina successiva
	  else
	    page=0;			//prima pagina
	}
	while(((((rowcount*page)+row)*rowlenchar)+col>((dati->eesize)-1))&&(col>0))	//se la riga non e' a lunghezza piena, potrebbe
	  col--;								//trovarsi fuori campo. Corregge
      }
      else if(ins==KEY_RIGHT)		//freccia dx
      {
	col++;				//colonna successiva
	if(col>(rowlenchar-1))
	{
	  col=0;			//prima colonna
	  if(row<rowcount-1)
	  {
	    row++;			//riga successiva
	  }
	  else
	  {
	    if((rowcount*page)+row<(rowcount-1))
	      row++;			//riga successiva
	    else
	    {
	      row=0;			//prima riga
	      if(page<(pages-1))
		page++;			//pagina successiva
	      else
		page=0;			//prima pagina
	    }
	  }
	}
	if((((rowcount*page)+row)*rowlenchar)+col>((dati->eesize)-1))
	{
	  row=0;
	  col=0;
	  page=0;
	}
      }
      else if(ins==KEY_LEFT)		//freccia sx
      {
	col--;				//colonna precedente
	if(col<0)
	{
	  col=rowlenchar-1;		//ultima colonna
	  if(row>0)
	  {
	    row--;			//riga precedente
	  }
	  else
	  {
	    if(page>0)
	      page--;			//pagina precedente
	    else
	    {
	      page=(pages-1);		//ultima pagina
	    }
	    row=ceil(((float)(dati->eesize)-(rowlenchar*rowcount*page))/rowlenchar)-1;
	    while(((rowlenchar*rowcount*page)+(row*rowlenchar)>(dati->eesize))||(row>rowcount-1))
	      row--;			//ultima riga della pagina
	  }
	  while(((((rowcount*page)+row)*rowlenchar)+col>((dati->eesize)-1))&&(col>0))	//se la riga non e' a lunghezza piena, potrebbe 
	    col--;								//trovarsi fuori campo. Corregge
	}
      }
      else if(ins==KEY_END)		//END
      {
	curs_set(0);			//esce dalla visualizzazione
	return;
      }
      else if(ins>31&&ins<127)		//gestisce solo i caratteri "normali"
      {
	dati->ee[((row*rowlenchar)+col)]=ins;	//modifica cella
	dati->eemod[((row*rowlenchar)+col)]=true;	//setta cella modificata
	if(col<rowlenchar-1)		//sposta il cursore in avanti
	  col++;			//colonna successiva
	else if(col==rowlenchar-1)
	{
	  col=0;			//prima colonna
	  if(row<((dati->eesize)/rowlenchar)-1)
	    row++;			//riga successiva
	  else
	    row=0;			//prima riga
	}
      }
   }
   curs_set(0);
   return;
}
