/*****************************************************************************
 *                                                                           *
 * PIC-232 - Comunicazione seriale per gestione memoria EEPROM 877 System    *
 * Copyright (C) 2009  by Walter Lain <kcswalter@member.fsf.org>             *
 *                                                                           *
 * This program is free software; you can redistribute it and/or modify      *
 * it under the terms of the GNU General Public License as published by      *
 * the Free Software Foundation; either version 3 of the License, or         *
 * (at your option) any later version.                                       *
 *                                                                           *
 * This program is distributed in the hope that it will be useful,           *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 * GNU General Public License for more details.                              *
 *                                                                           *
 * You should have received a copy of the GNU General Public License         *
 * along with this program; if not, write to the Free Software               *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
 *                                                                           *
 *****************************************************************************/

#include "params.h"			//definizioni programma


/********************************
 *           e2save()           *
 ********************************
 Salva il contenuto della EEPROM su file
 
Variabili esterne:
dati=struttura SYSDATA (vedere params.h)
strings=struttura SYSTEXT (vedere params.h)

Variabili interne:
choice=lettura input utente
fmem=oggetto file EEPROM
roff=offset riga
rows,cols=geometria schermo
*/
void e2save(SYSDATA *dati,SYSTEXT *strings)
{
   int roff,rows,cols;
   char choice;
   FILE *fmem;
   if(!dati->cmdline)			//se in modalita' grafica, chiede conferma
   {  
     clear();
     header(dati,strings);
     getmaxyx(stdscr, rows, cols);	//recupera la dimensione dello schermo
     if(rows<rowmin)
       rows=rowmin;
     if(cols<colmin)
       cols=colmin;
     roff=rows/2;				//centra in verticale
     mvprintw(roff-2,((cols-(strlen(strings->overwrite001)))/2),"%s",strings->overwrite001);//This will overwrite the content of file:
     mvprintw(roff,(((cols-6)-strlen(dati->memfile))/2),">>>%s<<<",dati->memfile);
     //ARE YOU SURE YOU WANT TO PROCEED? (yes/no)
     mvprintw(roff+2,((cols-(strlen(strings->sure001)+6))/2),"%s (%c/%c)",strings->sure001,strings->yes,strings->no);curs_set(0);
     refresh();
     while(true)
     {
	choice=getch();			//finche' l'utonto non preme yes o no resta qui
	if(choice==strings->no)
	   return;
	else if(choice==strings->yes)
	   break;
     }
   }
   else
   {
     roff=0;				//evita il warning di var non inizializzata in compilazione
   }
   fmem = fopen(dati->memfile,"w+");	//apre il file per salvare il contenuto EEPROM
   if(!fmem)
   {
      ioerror(dati,strings,dati->memfile);	//visualizza errore I/O
      return;
   }
   else
   {
      fwrite(dati->ee,sizeof(dati->ee[0]),dati->eesize,fmem);
   }
   fclose(fmem);
   if(dati->cmdline)			//messaggio ok in modalita' parametrica
   {
     printf("%i %s %s.\n",dati->eesize,strings->success001,dati->memfile);
   }
   else					//messaggio ok in modalita' grafica
   {
     clear();
     header(dati,strings);
     getmaxyx(stdscr, rows, cols);	//recupera la dimensione dello schermo
     if(rows<rowmin)
       rows=rowmin;
     if(cols<colmin)
       cols=colmin;
     //* bytes successfully written to file *
     mvprintw(roff-1,(((cols-(strlen(strings->success001)))-strlen(dati->memfile))/2),"%i %s %s.",dati->eesize,strings->success001,dati->memfile);
     mvprintw(roff+1,2,"%s",strings->strike001);	//Strike a key to continue...
     curs_set(0);
     refresh();
     getch();
   }
   return;
}


/********************************
 *           e2load()           *
 ********************************
 Carica il contenuto della EEPROM da file
 
Variabili esterne:
dati=struttura SYSDATA (vedere params.h)
strings=struttura SYSTEXT (vedere params.h)

Variabili interne:
buffer[]=buffer dinamico lettura file
choice=lettura input utente
dim=dimensione file
fmem=oggetto file EEPROM
n=contatore generico
rows,cols=geometria schermo
stFileInfo=struttura informazioni file
*/
int e2load(SYSDATA *dati,SYSTEXT *strings)
{
   int roff,dim=0,n,rows,cols;
   char choice,*buffer;
   struct stat stFileInfo;
   FILE *fmem;
   if(!dati->cmdline)			//se in modalita' grafica, chiede conferma
   {
     clear();
     header(dati,strings);
     getmaxyx(stdscr, rows, cols);	//recupera la dimensione dello schermo
     if(rows<rowmin)
       rows=rowmin;
     if(cols<colmin)
       cols=colmin;
     roff=rows/2;				//centra in verticale
     //This will change the local copy of EEPROM data with the content of file:
     mvprintw(roff-3,((cols-(strlen(strings->overwrite002)))/2),"%s",strings->overwrite002);
     mvprintw(roff-1,(((cols-6)-strlen(dati->memfile))/2),">>>%s<<<",dati->memfile);
     mvprintw(roff+1,((cols-(strlen(strings->overwrite003)))/2),"%s",strings->overwrite003);//(it will not affect the MCU EEPROM).
     //ARE YOU SURE YOU WANT TO PROCEED? (yes/no)
     mvprintw(roff+3,((cols-(strlen(strings->sure001)+6))/2),"%s (%c/%c)",strings->sure001,strings->yes,strings->no);curs_set(0);
     refresh();
     while(true)
     {
	choice=getch();			//finche' l'utonto non preme y o n resta qui
	if(choice==strings->no)
	{
	   return 1;
	}
	else if(choice==strings->yes)
	   break;
     }
   }
   else
   {
     roff=0;				//evita il warning di var non inizializzata in compilazione
     cols=0;
   }
   fmem = fopen(dati->memfile,"r");
   if(!fmem)
   {
      ioerror(dati,strings,dati->memfile);	//visualizza errore I/O
      return 1;
   }
   else
   {
      if(stat(dati->memfile, &stFileInfo))	//file non apribile
      {
	fclose(fmem);
	ioerror(dati,strings,dati->memfile);	//visualizza errore I/O
	return 1;
      }
      dim=stFileInfo.st_size;
      if(dim==0)			//file vuoto
      {
	fclose(fmem);
	ioerror(dati,strings,dati->memfile);	//visualizza errore I/O
	return 1;
      }
      else if(dim>(dati->eesize))		//file > dimensione eeprom
      {
	buffer=(char*)calloc(sizeof(char),dati->eesize);
	fread(buffer,sizeof(char),sizeof(char)*(dati->eesize),fmem);
      }
      else
      {
	buffer=(char*)calloc(sizeof(char),dim);
	fread(buffer,sizeof(char),sizeof(char)*(dim),fmem);
      }
      for(n=0;(n<(dati->eesize))&&(n<dim);n++)	//sistema dato
      {
	if(buffer[n]>255)
	  dati->ee[n]=255;
	else if(buffer[n]<0)
	  dati->ee[n]=0;
	else
	  dati->ee[n]=buffer[n];
	dati->eemod[n]=true; 		//aggiorna vettore modifiche
      }
   }
   fclose(fmem);
   free(buffer);
   if(dati->cmdline)			//messaggio ok in modalita' parametrica
   {
     printf("%i %s %s.\n",dim,strings->success002,dati->memfile);
   }
   else					//messaggio ok in modalita' grafica
   {
     clear();
     header(dati,strings);
     getmaxyx(stdscr, rows, cols);	//recupera la dimensione dello schermo
     if(rows<rowmin)
       rows=rowmin;
     if(cols<colmin)
       cols=colmin;
     //* bytes successfully loaded from file *
     mvprintw(roff-2,(((cols-(strlen(strings->success002)+6))-strlen(dati->memfile))/2),"%i %s %s.",dim,strings->success002,dati->memfile);
   }
   if(dim<(dati->eesize))		//file < size
   {
     if(dati->cmdline)			//messaggio errore in modalita' parametrica
     {
       printf("%s %i bytes, %s %i.\n\n",strings->eesize001,dati->eesize,strings->eesize002,dim);//EEPROM size is set to * bytes, but the file contains only *
     }
     else				//messaggio errore in modalita' grafica
     {
       //EEPROM size is set to * bytes, but the file contains only *
       mvprintw(roff,((cols-(strlen(strings->eesize001)+strlen(strings->eesize002)+16))/2),"%s %i bytes, %s %i.",strings->eesize001,dati->eesize,strings->eesize002,dim);
       mvprintw(roff+1,((cols-(strlen(strings->setchk001)))/2),"%s",strings->setchk001);//You should check your settings.
       for(n=dim;n<(dati->eesize);n++)	//pulisce il resto del vettore
	 dati->ee[n]=0;
     }
   }
   else if(dim>(dati->eesize))		//file > size
   {
     if(dati->cmdline)			//messaggio errore in modalita' parametrica
     {
       printf("%s %i bytes, %s %i.\n\n",strings->eesize003,dim,strings->eesize004,dati->eesize);//The file contains * bytes, but EEPROM size is set to only *
     }
     else				//messaggio errore in modalita' grafica
     {
       //The file contains * bytes, but EEPROM size is set to only *
       mvprintw(roff,((cols-(strlen(strings->eesize003)+strlen(strings->eesize004)+16))/2),"%s %i bytes, %s %i.",strings->eesize003,dim,strings->eesize004,dati->eesize);
       mvprintw(roff+1,((cols-(strlen(strings->setchk001)))/2),"%s",strings->setchk001);//You should check your settings.
     }
   }
   if(!dati->cmdline)			//refresh modalita' grafica
   {
     mvprintw(roff+3,2,"%s",strings->strike001);	//Strike a key to continue...
     curs_set(0);
     refresh();
     getch();
   }
   return 0;
}


/********************************
 *          readcfg()           *
 ********************************
 Legge configurazione dal file
 
Variabili esterne:
dati=struttura SYSDATA (vedere params.h)
strings=struttura SYSTEXT (vedere params.h)

Variabili interne:
buffer[]=buffer dinamico lettura file
dim=dimensione file
fcfg=oggetto file configurazione
len=dimensione buffer lettura
n=contatore generico
pointer=puntatore recupero dati
*/
int readcfg(SYSDATA *dati,SYSTEXT *strings)
{
   char *buffer;
   int pointer,n,dim;
   size_t len=0;
   FILE *fcfg;
   fcfg = fopen(cfgfile,"r");
   if(!fcfg)
   {
      ioerror(dati,strings,cfgfile); 	//visualizza errore I/O
      return 1;
   }
   else
   {
      while(!feof(fcfg))
      {
	 len=0;
	 dim=getline(&buffer,&len,fcfg);	//legge una riga del file
	 if(strlen(buffer)==0)			//riga vuota
	    ;
	 else if(buffer[0]=='#')		//commento
	    free(buffer);
	 else if(strstr(buffer,"port")!=0)	//recupera porta seriale
	 {
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='=')
	    {
	      pointer++;
	    }
	    do
	    {
	      pointer++;
	    }while(pointer<(dim-1)&&buffer[pointer]!='/');
	    for(n=0;n<(dim-pointer-1)&&(dim-pointer)<maxport;n++)
	       dati->port[n]=buffer[n+pointer];
	    dati->port[n]='\0';
	    free(buffer);
	    if(DEBUG)
	    {
	      if(dati->cmdline)
		printf("   Serial port: %s\n",dati->port);
	      else
		printw("   Serial port: %s\n",dati->port);
	    }
	 }
	 else if(strstr(buffer,"baud")!=0)	//recupera baud rate
	 {
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='=')
	    {
	      pointer++;
	    }
	    do
	    {
	      pointer++;
	    }while(pointer<(dim-1)&&(buffer[pointer]<'0'||buffer[pointer]>'9'));
	    if(pointer>=(dim-1))	//se non trova il dato, imposta default
	    {
	      dati->speed=19200;
	    }
	    else
	    {
	      dati->speed=0;
	      for(n=0;(n+pointer)<(dim-1);n++)
	      {
		(dati->speed)+=(buffer[n+pointer]-48)*pow(10,((dim-1)-(n+pointer)-1));
	      }
	    }
	    free(buffer);
	    if(DEBUG)
	    {
	      if(dati->cmdline)
		printf("   BAUD rate: %i\n",dati->speed);
	      else
		printw("   BAUD rate: %i\n",dati->speed);
	    }
	 }
	 else if(strstr(buffer,"size")!=0)	//recupera dimensione eeprom
	 {
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='=')
	    {
	      pointer++;
	    }
	    do
	    {
	      pointer++;
	    }while(pointer<(dim-1)&&(buffer[pointer]<'0'||buffer[pointer]>'9'));
	    if(pointer>=(dim-1))	//se non trova il dato, imposta default
	    {
	      dati->eesize=64;
	    }
	    else
	    {
	      dati->eesize=0;
	      for(n=0;(n+pointer)<(dim-1);n++)
	      {
		(dati->eesize)+=(buffer[n+pointer]-48)*pow(10,((dim-1)-(n+pointer)-1));
	      }
	    }
	    free(buffer);
	    if(DEBUG)
	    {
	      if(dati->cmdline)
		printf("   EEPROM size: %i\n",dati->eesize);
	      else
		printw("   EEPROM size: %i\n",dati->eesize);
	    }
	 }
	 else if(strstr(buffer,"mode")!=0)	//recupera modo scrittura
	 {
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='=')
	    {
	      pointer++;
	    }
	    do
	    {
	      pointer++;
	    }while(pointer<(dim-1)&&(buffer[pointer]<'a'||buffer[pointer]>'z'));
	    if(buffer[pointer]=='m'||buffer[pointer]=='o')	//se il dato non corrisponde, imposta default
	    {
	      dati->eemode=buffer[pointer];
	    }
	    else
	    {
	      dati->eemode='m';
	    }
	    free(buffer);
	    if(DEBUG)
	    {
	      if(dati->cmdline)
		printf("   EEPROM write mode: %c\n",dati->eemode);
	      else
		printw("   EEPROM write mode: %c\n",dati->eemode);
	    }
	 }
	 else if(strstr(buffer,"data")!=0)	//recupera modo visualizzazione
	 {
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='=')
	    {
	      pointer++;
	    }
	    do
	    {
	      pointer++;
	    }
	    while(pointer<(dim-1)&&(buffer[pointer]<'a'||buffer[pointer]>'z'));
	    if(buffer[pointer]=='c'||buffer[pointer]=='n')	//se il dato non corrisponde, imposta default
	    {
	      dati->eedata=buffer[pointer];
	    }
	    else
	    {
	      dati->eedata='c';
	    }
	    free(buffer);
	    if(DEBUG)
	    {
	      if(dati->cmdline)
		printf("   EEPROM data mode: %c\n",dati->eedata);
	      else
		printw("   EEPROM data mode: %c\n",dati->eedata);
	    }
	 }
	 else if(strstr(buffer,"file")!=0)	//recupera nome file eeprom
	 {
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='=')
	    {
	      pointer++;
	    }
	    do
	    {
	      pointer++;
	    }
	    while(pointer<(dim-1)&&buffer[pointer]==' ');
	    if(pointer>=(dim-1))	//se non trova il dato, azzera il vettore
	    {
	      if(dati->memfile!=NULL)
		free(dati->memfile);
	      dati->memfile=(char*)calloc(sizeof(char),1);
	      dati->memfile[0]='\0';
	    }
	    else
	    {
	      if(dati->memfile!=NULL)
		free(dati->memfile);
	      dati->memfile=(char*)calloc(sizeof(char),(dim-pointer));//rialloca la memoria necessaria
	      for(n=0;(n<(dim-pointer-1));n++)
	      {
		dati->memfile[n]=buffer[n+pointer];
	      }
	      dati->memfile[n]='\0';
	    }
	    free(buffer);
	    if(DEBUG)
	    {
	      if(dati->cmdline)
		printf("   EEPROM file: %s\n",dati->memfile);
	      else
		printw("   EEPROM file: %s\n",dati->memfile);
	    }
	 }
      }
   }
   fclose(fcfg);
   free(buffer);
   if(DEBUG&&!dati->cmdline)
   {
     refresh();
     getch();
   }
   return 0;
}


/********************************
 *          savecfg()           *
 ********************************
 Salva configurazione sul file
 
Variabili esterne:
dati=struttura SYSDATA (vedere params.h)
strings=struttura SYSTEXT (vedere params.h)

Variabili interne:
choice=lettura input utente
fcfg=oggetto file configurazione
roff=offset riga
rows,cols=geometria schermo
*/
int savecfg(SYSDATA *dati,SYSTEXT *strings)
{
   char choice;
   FILE *fcfg;
   int roff,rows,cols;

   clear();
   header(dati,strings);
   getmaxyx(stdscr, rows, cols);	//recupera la dimensione dello schermo
   if(rows<rowmin)
     rows=rowmin;
   if(cols<colmin)
     cols=colmin;
   roff=rows/2;				//centra in verticale
   attron(A_BOLD);
   mvprintw(roff-1,(cols-(strlen(strings->overwrite004)))/2,"%s",strings->overwrite004);//This will overwrite the configuration file.
   attroff(A_BOLD);
   //ARE YOU SURE YOU WANT TO PROCEED? (yes/no)
   mvprintw(roff+1,((cols-(strlen(strings->sure001)+6))/2),"%s (%c/%c)",strings->sure001,strings->yes,strings->no);
   curs_set(0);
   refresh();
   while(true)
   {
      choice=getch();			//finche' l'utonto non preme y o n resta qui
      if(choice==strings->no)
	 return 0;
      else if(choice==strings->yes)
	 break;
   }
   fcfg = fopen(cfgfile,"w+");		//apre il file per salvare i parametri
   if(!fcfg)
   {
      ioerror(dati,strings,cfgfile);	//visualizza errore I/O
      return 1;
   }
   else
   {
     fprintf(fcfg,"# PIC-232 config parameters\n");
     fprintf(fcfg,"# the order of parameters can be changed freely\n");
     fprintf(fcfg,"# to comment use \"#\" \n\n");
     fprintf(fcfg,"# Serial port:\n");
     fprintf(fcfg,"port=%s\n\n",dati->port);
     fprintf(fcfg,"# Baud rate:\n");
     fprintf(fcfg,"baud=%i\n\n",dati->speed);
     fprintf(fcfg,"# EEPROM size:\n");
     fprintf(fcfg,"size=%i\n\n",dati->eesize);
     fprintf(fcfg,"# EEPROM write mode (m=Modified cells only - o=Overwrite all):\n");
     fprintf(fcfg,"mode=%c\n\n",dati->eemode);
     fprintf(fcfg,"# EEPROM data mode (c=Character display - n=Integer display):\n");
     fprintf(fcfg,"data=%c\n\n",dati->eedata);
     fprintf(fcfg,"# EEPROM file (no space allowed in the file name):\n");
     fprintf(fcfg,"file=%s\n\n",dati->memfile);
   }
   fclose(fcfg);
   clear();
   header(dati,strings);
   getmaxyx(stdscr, rows, cols);	//recupera la dimensione dello schermo
   if(rows<rowmin)
     rows=rowmin;
   if(cols<colmin)
     cols=colmin;
   mvprintw(roff-1,(cols-(strlen(strings->success003)))/2,"%s",strings->success003);//Configuration saved successfully.
   mvprintw(roff+1,2,"%s",strings->strike001);	//Strike a key to continue...
   curs_set(0);
   refresh();
   getch();
   return 0;
}

/********************************
 *         readlocal()          *
 ********************************
 Carica stringhe di localizzazione
 
Variabili esterne:
strings=struttura SYSTEXT (vedere params.h)

Variabili interne:
buffer[]=buffer dinamico lettura file
dim=dimensione file
fstr=oggetto file localizzazione
len=dimensione buffer lettura
n=contatore generico
pointer=puntatore recupero dati
roff=offset riga
rows,cols=geometria schermo
*/
int readlocal(bool cmdline,SYSTEXT *strings)
{
   char *buffer;
   int pointer,n,dim,roff,rows,cols;
   size_t len=0;
   FILE *fstr;
   fstr = fopen(locfile,"r");
   if(!fstr)
   {
      if(cmdline)		//messaggio di errore in modalita' parametrica
      {
	printf("I/O error.\n");
	printf("Localization data not found.\n");
	printf("The program will be terminated.\n\n");
      }
      else				//messaggio di errore in modalita' grafica
      {
	clear();
	getmaxyx(stdscr, rows, cols);	//recupera la dimensione dello schermo
	if(rows<rowmin)
	  rows=rowmin;
	if(cols<colmin)
	  cols=colmin;
	roff=rows/2;			//centra in verticale
	attron(A_BOLD);
	mvprintw(roff-3,(cols-10)/2,"I/O error.");
	mvprintw(roff-1,(cols-28)/2,"Localization data not found.");
	mvprintw(roff+1,(cols-31)/2,"The program will be terminated.");
	attroff(A_BOLD);
	mvprintw(roff+3,2,"Strike a key to exit...");
	curs_set(0);
	refresh();
	getch();
      }
      return 1;
   }
   else
   {
      while(!feof(fstr))
      {
	 len=0;
	 dim=getline(&buffer,&len,fstr);	//legge una riga del file
	 if(strlen(buffer)==0)			//riga vuota
	    ;
	 else if(buffer[0]=='#')		//commento
	    free(buffer);
	 else if(strstr(buffer,"yes#")!=0)
	 {
	    strings->yes=buffer[4];
	    free(buffer);
	 }
	 else if(strstr(buffer,"no#")!=0)
	 {
	    strings->no=buffer[3];
	    free(buffer);
	 }
	 else if(strstr(buffer,"eesize001")!=0)
	 {
	    strings->eesize001=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->eesize001[n]=buffer[n+pointer];
	    strings->eesize001[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"eesize002")!=0)
	 {
	    strings->eesize002=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->eesize002[n]=buffer[n+pointer];
	    strings->eesize002[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"eesize003")!=0)
	 {
	    strings->eesize003=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->eesize003[n]=buffer[n+pointer];
	    strings->eesize003[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"eesize004")!=0)
	 {
	    strings->eesize004=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->eesize004[n]=buffer[n+pointer];
	    strings->eesize004[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"eesize005")!=0)
	 {
	    strings->eesize005=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->eesize005[n]=buffer[n+pointer];
	    strings->eesize005[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"eesize006")!=0)
	 {
	    strings->eesize006=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->eesize006[n]=buffer[n+pointer];
	    strings->eesize006[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"eesize007")!=0)
	 {
	    strings->eesize007=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->eesize007[n]=buffer[n+pointer];
	    strings->eesize007[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"err001")!=0)
	 {
	    strings->err001=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->err001[n]=buffer[n+pointer];
	    strings->err001[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"err002")!=0)
	 {
	    strings->err002=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->err002[n]=buffer[n+pointer];
	    strings->err002[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"err003")!=0)
	 {
	    strings->err003=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->err003[n]=buffer[n+pointer];
	    strings->err003[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"err004")!=0)
	 {
	    strings->err004=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->err004[n]=buffer[n+pointer];
	    strings->err004[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"err005")!=0)
	 {
	    strings->err005=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->err005[n]=buffer[n+pointer];
	    strings->err005[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"err006")!=0)
	 {
	    strings->err006=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->err006[n]=buffer[n+pointer];
	    strings->err006[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"header001")!=0)
	 {
	    strings->header001=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->header001[n]=buffer[n+pointer];
	    strings->header001[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"header002")!=0)
	 {
	    strings->header002=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->header002[n]=buffer[n+pointer];
	    strings->header002[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"header003")!=0)
	 {
	    strings->header003=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->header003[n]=buffer[n+pointer];
	    strings->header003[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"header004")!=0)
	 {
	    strings->header004=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->header004[n]=buffer[n+pointer];
	    strings->header004[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"header005")!=0)
	 {
	    strings->header005=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->header005[n]=buffer[n+pointer];
	    strings->header005[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"header006")!=0)
	 {
	    strings->header006=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->header006[n]=buffer[n+pointer];
	    strings->header006[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"header007")!=0)
	 {
	    strings->header007=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->header007[n]=buffer[n+pointer];
	    strings->header007[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"info001")!=0)
	 {
	    strings->info001=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->info001[n]=buffer[n+pointer];
	    strings->info001[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"menu001")!=0)
	 {
	    strings->menu001=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->menu001[n]=buffer[n+pointer];
	    strings->menu001[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"menu002")!=0)
	 {
	    strings->menu002=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->menu002[n]=buffer[n+pointer];
	    strings->menu002[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"menu003")!=0)
	 {
	    strings->menu003=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->menu003[n]=buffer[n+pointer];
	    strings->menu003[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"menu004")!=0)
	 {
	    strings->menu004=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->menu004[n]=buffer[n+pointer];
	    strings->menu004[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"menu005")!=0)
	 {
	    strings->menu005=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->menu005[n]=buffer[n+pointer];
	    strings->menu005[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"menu006")!=0)
	 {
	    strings->menu006=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->menu006[n]=buffer[n+pointer];
	    strings->menu006[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"menu007")!=0)
	 {
	    strings->menu007=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->menu007[n]=buffer[n+pointer];
	    strings->menu007[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"menu008")!=0)
	 {
	    strings->menu008=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->menu008[n]=buffer[n+pointer];
	    strings->menu008[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"menu009")!=0)
	 {
	    strings->menu009=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->menu009[n]=buffer[n+pointer];
	    strings->menu009[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"menu010")!=0)
	 {
	    strings->menu010=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->menu010[n]=buffer[n+pointer];
	    strings->menu010[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"menu020")!=0)
	 {
	    strings->menu020=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->menu020[n]=buffer[n+pointer];
	    strings->menu020[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"menu021")!=0)
	 {
	    strings->menu021=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->menu021[n]=buffer[n+pointer];
	    strings->menu021[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"menu022")!=0)
	 {
	    strings->menu022=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->menu022[n]=buffer[n+pointer];
	    strings->menu022[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"menu023")!=0)
	 {
	    strings->menu023=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->menu023[n]=buffer[n+pointer];
	    strings->menu023[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"menu024")!=0)
	 {
	    strings->menu024=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->menu024[n]=buffer[n+pointer];
	    strings->menu024[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"menu025")!=0)
	 {
	    strings->menu025=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->menu025[n]=buffer[n+pointer];
	    strings->menu025[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"menu030")!=0)
	 {
	    strings->menu030=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->menu030[n]=buffer[n+pointer];
	    strings->menu030[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"menu031")!=0)
	 {
	    strings->menu031=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->menu031[n]=buffer[n+pointer];
	    strings->menu031[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"menu032")!=0)
	 {
	    strings->menu032=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->menu032[n]=buffer[n+pointer];
	    strings->menu032[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"menu040")!=0)
	 {
	    strings->menu040=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->menu040[n]=buffer[n+pointer];
	    strings->menu040[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"menu041")!=0)
	 {
	    strings->menu041=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->menu041[n]=buffer[n+pointer];
	    strings->menu041[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"menu042")!=0)
	 {
	    strings->menu042=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->menu042[n]=buffer[n+pointer];
	    strings->menu042[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"menu043")!=0)
	 {
	    strings->menu043=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->menu043[n]=buffer[n+pointer];
	    strings->menu043[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"menu050")!=0)
	 {
	    strings->menu050=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->menu050[n]=buffer[n+pointer];
	    strings->menu050[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"menu051")!=0)
	 {
	    strings->menu051=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->menu051[n]=buffer[n+pointer];
	    strings->menu051[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"overwrite001")!=0)
	 {
	    strings->overwrite001=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->overwrite001[n]=buffer[n+pointer];
	    strings->overwrite001[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"overwrite002")!=0)
	 {
	    strings->overwrite002=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->overwrite002[n]=buffer[n+pointer];
	    strings->overwrite002[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"overwrite003")!=0)
	 {
	    strings->overwrite003=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->overwrite003[n]=buffer[n+pointer];
	    strings->overwrite003[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"overwrite004")!=0)
	 {
	    strings->overwrite004=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->overwrite004[n]=buffer[n+pointer];
	    strings->overwrite004[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"overwrite005")!=0)
	 {
	    strings->overwrite005=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->overwrite005[n]=buffer[n+pointer];
	    strings->overwrite005[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"overwrite006")!=0)
	 {
	    strings->overwrite006=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->overwrite006[n]=buffer[n+pointer];
	    strings->overwrite006[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"overwrite007")!=0)
	 {
	    strings->overwrite007=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->overwrite007[n]=buffer[n+pointer];
	    strings->overwrite007[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"overwrite008")!=0)
	 {
	    strings->overwrite008=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->overwrite008[n]=buffer[n+pointer];
	    strings->overwrite008[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"pgupdown001")!=0)
	 {
	    strings->pgupdown001=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->pgupdown001[n]=buffer[n+pointer];
	    strings->pgupdown001[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"setchk001")!=0)
	 {
	    strings->setchk001=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->setchk001[n]=buffer[n+pointer];
	    strings->setchk001[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"setchk002")!=0)
	 {
	    strings->setchk002=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->setchk002[n]=buffer[n+pointer];
	    strings->setchk002[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"splash001")!=0)
	 {
	    strings->splash001=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->splash001[n]=buffer[n+pointer];
	    strings->splash001[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"str001")!=0)
	 {
	    strings->str001=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->str001[n]=buffer[n+pointer];
	    strings->str001[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"str002")!=0)
	 {
	    strings->str002=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->str002[n]=buffer[n+pointer];
	    strings->str002[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"str003")!=0)
	 {
	    strings->str003=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->str003[n]=buffer[n+pointer];
	    strings->str003[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"str004")!=0)
	 {
	    strings->str004=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->str004[n]=buffer[n+pointer];
	    strings->str004[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"str005")!=0)
	 {
	    strings->str005=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->str005[n]=buffer[n+pointer];
	    strings->str005[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"str006")!=0)
	 {
	    strings->str006=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->str006[n]=buffer[n+pointer];
	    strings->str006[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"str007")!=0)
	 {
	    strings->str007=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->str007[n]=buffer[n+pointer];
	    strings->str007[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"str008")!=0)
	 {
	    strings->str008=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->str008[n]=buffer[n+pointer];
	    strings->str008[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"str009")!=0)
	 {
	    strings->str009=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->str009[n]=buffer[n+pointer];
	    strings->str009[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"str010")!=0)
	 {
	    strings->str010=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->str010[n]=buffer[n+pointer];
	    strings->str010[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"str011")!=0)
	 {
	    strings->str011=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->str011[n]=buffer[n+pointer];
	    strings->str011[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"str012")!=0)
	 {
	    strings->str012=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->str012[n]=buffer[n+pointer];
	    strings->str012[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"str013")!=0)
	 {
	    strings->str013=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->str013[n]=buffer[n+pointer];
	    strings->str013[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"str014")!=0)
	 {
	    strings->str014=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->str014[n]=buffer[n+pointer];
	    strings->str014[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"str015")!=0)
	 {
	    strings->str015=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->str015[n]=buffer[n+pointer];
	    strings->str015[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"str016")!=0)
	 {
	    strings->str016=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->str016[n]=buffer[n+pointer];
	    strings->str016[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"str017")!=0)
	 {
	    strings->str017=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->str017[n]=buffer[n+pointer];
	    strings->str017[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"str018")!=0)
	 {
	    strings->str018=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->str018[n]=buffer[n+pointer];
	    strings->str018[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"str019")!=0)
	 {
	    strings->str019=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->str019[n]=buffer[n+pointer];
	    strings->str019[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"str020")!=0)
	 {
	    strings->str020=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->str020[n]=buffer[n+pointer];
	    strings->str020[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"strike001")!=0)
	 {
	    strings->strike001=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->strike001[n]=buffer[n+pointer];
	    strings->strike001[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"success001")!=0)
	 {
	    strings->success001=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->success001[n]=buffer[n+pointer];
	    strings->success001[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"success002")!=0)
	 {
	    strings->success002=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->success002[n]=buffer[n+pointer];
	    strings->success002[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"success003")!=0)
	 {
	    strings->success003=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->success003[n]=buffer[n+pointer];
	    strings->success003[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"success004")!=0)
	 {
	    strings->success004=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->success004[n]=buffer[n+pointer];
	    strings->success004[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"success005")!=0)
	 {
	    strings->success005=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->success005[n]=buffer[n+pointer];
	    strings->success005[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"success006")!=0)
	 {
	    strings->success006=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->success006[n]=buffer[n+pointer];
	    strings->success006[n]='\0';
	    free(buffer);
	 }
	 else if(strstr(buffer,"sure001")!=0)
	 {
	    strings->sure001=malloc(sizeof(char)*dim);
	    pointer=0;
	    while(pointer<(dim-1)&&buffer[pointer]!='#')
	    {
	      pointer++;
	    }
	    pointer++;
	    for(n=0;n<(dim-pointer-1);n++)
	       strings->sure001[n]=buffer[n+pointer];
	    strings->sure001[n]='\0';
	    free(buffer);
	 }
      }
   }
   fclose(fstr);
   free(buffer);
   return 0;
}
