/*****************************************************************************
 *                                                                           *
 * PIC-232 - Comunicazione seriale per gestione memoria EEPROM 877 System    *
 * Copyright (C) 2009  by Walter Lain <kcswalter@member.fsf.org>             *
 *                                                                           *
 * This program is free software; you can redistribute it and/or modify      *
 * it under the terms of the GNU General Public License as published by      *
 * the Free Software Foundation; either version 3 of the License, or         *
 * (at your option) any later version.                                       *
 *                                                                           *
 * This program is distributed in the hope that it will be useful,           *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 * GNU General Public License for more details.                              *
 *                                                                           *
 * You should have received a copy of the GNU General Public License         *
 * along with this program; if not, write to the Free Software               *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
 *                                                                           *
 *****************************************************************************/

#include "params.h"			//definizioni programma


/********************************
 *           splash()           *
 ********************************
 Splash screen

Variabili esterne:
n/a

Variabili interne:
i=contatore generico
roff=offset riga
rows,cols=geometria schermo
*/
void splash(bool cmdline,SYSTEXT *strings)
{
   int i=0,roff,rows,cols;
   clear();
   if(cmdline)				//se avviato in modalita' parametrica, mostra un semplice testo
   {
     system("clear");
     printf("PIC-232 v%i.%i%c Copyright (C) %i Walter Lain\n",rev1,rev2,rev3,year);
     printf("PIC-232 comes with ABSOLUTELY NO WARRANTY\n");
     printf("This is free software, and you are welcome to redistribute it under certain conditions\n");
     printf("See source files for more details\n\n");
   }
   else					//altrimenti mostra lo splash
   {
     getmaxyx(stdscr, rows, cols);	//recupera la dimensione dello schermo
     if(rows<rowmin)
       rows=rowmin;
     if(cols<colmin)
       cols=colmin;
     roff=((rows-18)/2);			//centra in verticale
     refresh();
     attron(A_BOLD);
     mvprintw(roff+2,((cols-44)/2),"PIC-232 v%i.%i%c Copyright (C) %i Walter Lain",rev1,rev2,rev3,year);
     mvprintw(roff+5,((cols-41)/2),"PIC-232 comes with ABSOLUTELY NO WARRANTY");
     mvprintw(roff+8,((cols-42)/2),"This is free software, and you are welcome");
     mvprintw(roff+9,((cols-43)/2),"to redistribute it under certain conditions");
     mvprintw(roff+12,((cols-33)/2),"See source files for more details");
     attron(A_BLINK);
     mvprintw(roff+15,((cols-(strlen(strings->splash001)))/2),"%s",strings->splash001);	//PRESS ENTER TO PROCEED
     attroff(A_BLINK);
     mvaddch(roff,((cols-50)/2),ACS_ULCORNER);//disegna riquadro
     for(i=0;i<48;i++)
       addch(ACS_HLINE);
     addch(ACS_URCORNER);
     for(i=roff+1;i<(roff+17);i++)
     {
       mvaddch(i,((cols-50)/2),ACS_VLINE);
       mvaddch(i,((cols-50)/2)+49,ACS_VLINE);
     }
     mvaddch(roff+17,((cols-50)/2),ACS_LLCORNER);
     for(i=0;i<48;i++)
       addch(ACS_HLINE);
     addch(ACS_LRCORNER);
     attroff(A_BOLD);
     curs_set(0);
     refresh();
     getch();
   }
   return;
}


/********************************
 *           header()           *
 ********************************
 Intestazione
 
Variabili esterne:
dati=struttura SYSDATA (vedere params.h)

Variabili interne:
i=contatore generico
j=posizione cursore riga
n=contatore celle modificate
ptemp=nome porta seriale
rows,cols=geometria schermo
*/
void header(SYSDATA *dati,SYSTEXT *strings)
{
   int i=0,j=0,n=0,rows,cols;
   char ptemp[11];
   clear();
   attron(A_BOLD|A_STANDOUT|A_REVERSE);
   getmaxyx(stdscr, rows, cols);	//recupera la dimensione dello schermo
   if(rows<rowmin)
     rows=rowmin;
   if(cols<colmin)
     cols=colmin;
   for(i=0;i<(dati->eesize);i++)	//conta le celle modificate
   {
     if(dati->eemod[i])
       n++;
   }
   mvaddch(0,0,ACS_ULCORNER);		//disegna riquadro
   for(i=0;i<(cols-2);i++)
     addch(ACS_HLINE);
   addch(ACS_URCORNER);
   mvaddch(2,0,ACS_LLCORNER);
   for(i=0;i<(cols-2);i++)
     addch(ACS_HLINE);
   addch(ACS_LRCORNER);
   mvaddch(1,0,ACS_VLINE);
   mvaddch(1,(cols-1),ACS_VLINE);
   move(1,1);
   for(i=0;i<((cols-46)/2);i++)
     addch(' ');
   if(date<1000)
     mvprintw(1,(cols-46)/2,"PIC-232 - by Walter Lain %i - r%i.%i%c %i0%i ",year,rev1,rev2,rev3,year,date);	//mesi da 1 a 9
   else
     mvprintw(1,(cols-46)/2,"PIC-232 - by Walter Lain %i - r%i.%i%c %i%i ",year,rev1,rev2,rev3,year,date);	//mesi da 10 a 12
   getyx(stdscr,j,i);			//recupera la posizione del cursore
   for(;i<(cols-1);i++) 		//aggiunge spazi fino in fondo
     addch(' ');
   attroff(A_BOLD|A_STANDOUT|A_REVERSE);
//          <ttyUSB10>  EEPROM - Size: 1234 - Mode: MOD - Data: CHR - Modified cells: 1234
   mvprintw(3,(cols-78)/2,"<");
   if(dati->serPort==-1)		//se porta non va, visualizza "errore"
   {
     attron(A_BOLD | A_BLINK);
     printw("%s",strings->header001);	//error
     attroff(A_BOLD | A_BLINK);
     printw(">"); 
   }
   else					//altrimenti visualizza nome porta
   {
      j=0;
      for(j=0;j<10&&(dati->port[j+5])!='\0';j++)
	ptemp[j]=dati->port[j+5];
      ptemp[j]='\0';
      printw("%s>",ptemp);
   }
   mvprintw(3,((cols-78)/2)+12,"EEPROM - %s: %i",strings->header002,dati->eesize);//EEPROM - Size
   if((dati->eesize)>999)		//aggiunge spazi
     ;
   else if((dati->eesize)>99)
     printw(" ");
   else
     printw("  ");
   printw(" - %s: ",strings->header003);	//Mode:
   if((dati->eemode)=='m')
     printw("MOD - ");
   else
     printw("ALL - ");
   printw("%s: ",strings->header004);	//Data:
   if((dati->eedata)=='c')
     printw("%s - ",strings->header005);	//CHR -
   else
     printw("%s - ",strings->header006);	//NUM -
   printw("%s: ",strings->header007);	//Modified cells:
   printw("%i",n);
   curs_set(0);
   refresh();
   return;
}


/********************************
 *            menu()            *
 ********************************
 Menu' funzioni
 
Variabili esterne:
dati=struttura SYSDATA (vedere params.h)

Variabili interne:
coff=offset colonna
rows,cols=geometria schermo
select=selezione funzione menu
serPort=oggetto porta seriale
*/
int menu(SYSDATA *dati,SYSTEXT *strings)
{
   int coff,select,rows,cols;
   if(!fileexists(dati->port))		//verifica se la porta default esiste
     dati->serPort=-1;			//no, imposta valore porta a -1 (serve per header())
   else
     dati->serPort=0;			//si, imposta valore porta a 0 (serve per header())
   clear();
   header(dati,strings);
   getmaxyx(stdscr, rows, cols);	//recupera la dimensione dello schermo
   if(rows<rowmin)
     rows=rowmin;
   if(cols<colmin)
     cols=colmin;
   coff=(cols/2)-10;			//il menu' viene centrato in orizzontale, ma non in verticale

   mvprintw(6,coff,"1) %s",strings->menu001);	//READ MEMORY
   mvprintw(7,coff,"2) %s",strings->menu002);	//WRITE MEMORY
   
   mvprintw(9,coff,"3) %s",strings->menu003);	//SHOW MEMORY
   mvprintw(10,coff,"4) %s",strings->menu004);	//EDIT MEMORY

   mvprintw(12,coff,"5) %s",strings->menu005);	//SAVE TO FILE
   mvprintw(13,coff,"6) %s",strings->menu006);	//LOAD FROM FILE

   mvprintw(15,coff,"7) %s",strings->menu007);	//PROGRAM SETTINGS

   mvprintw(17,coff,"8) %s",strings->menu008);	//PING TEST

   mvprintw(19,coff,"?) %s",strings->menu009);	//INFO SCREEN

   mvprintw(21,coff,"9) %s",strings->menu010);	//EXIT
   curs_set(0);
   refresh();
   select=getch();
   switch(select)
   {
      case '1':				//legge eeprom mcu
      {
	e2read(dati,strings,0);
	break;
      }
      case '2':				//scrive eeprom mcu
      {
	e2write(dati,strings);
	break;
      }
      case '3':				//visualizza eeprom
      {
	if((dati->eedata)=='c')
	  e2showchar(dati,strings);
	else
	  e2shownum(dati,strings);
	break;
      }
      case '4':				//modifica eeprom
      {
	if((dati->eedata)=='c')
	  e2modchar(dati,strings);
	else
	  e2modnum(dati,strings);
	break;
      }
      case '5':				//salva eeprom su file
      {
	e2save(dati,strings);
	break;
      }
      case '6':				//carica eeprom da file
      {
	e2load(dati,strings);
	break;
      }
      case '7':				//menu configurazione
      {
	setprog(dati,strings);
	break;
      }
      case '8':				//verifica comunicazione seriale
      {
	ping(dati,strings);
	break;
      }
      case '?':				//schermata informazioni
      {
	info(dati,strings);
	break;
      }
      case '9':				//uscita
      {
	clear();
 	return 1;
	break;
      }
      default:
      {
	break;
      }
   }
   return 0;
}


/********************************
 *          setprog()           *
 ********************************
 Selezione parametri programma
 
Variabili esterne:
dati=struttura SYSDATA (vedere params.h)

Variabili interne:
choice=lettura input utente
coff=offset colonna
roff=offset riga
rows,cols=geometria schermo
select=selezione funzione menu
*/
void setprog(SYSDATA *dati,SYSTEXT *strings)
{
  int coff,roff,select,rows,cols;
  char choice;
  while(true)
  {
    clear();
    header(dati,strings);
    getmaxyx(stdscr, rows, cols);	//recupera la dimensione dello schermo
    if(rows<rowmin)
      rows=rowmin;
    if(cols<colmin)
      cols=colmin;
    coff=(cols/2)-10;			//il menu' viene centrato in orizzontale, ma non in verticale
    attron(A_BOLD);
    mvprintw(6,((cols-(strlen(strings->menu020)))/2)+1,"%s",strings->menu020);	//PROGRAM SETTINGS
    attroff(A_BOLD);
    
    mvprintw(8,coff,"1) %s",strings->menu021);	//Serial Port parameters

    mvprintw(10,coff,"2) %s",strings->menu022);	//EEPROM parameters

    mvprintw(12,coff,"3) %s",strings->menu023);	//File I/O parameters

    mvprintw(14,coff,"4) %s",strings->menu024);	//Save config to file

    mvprintw(16,coff,"5) %s",strings->menu025);	//Load config from file

    mvprintw(21,coff,"9) %s",strings->menu010);	//EXIT
    curs_set(0);
    refresh();
    select=getch();
    switch(select)
    {
      case '1':				//configurazione porta seriale
      {
	sercfg(dati,strings);
	break;
      }
      case '2':				//configurazione eeprom
      {
	memcfg(dati,strings);
	break;
      }
      case '3':				//configurazione file i/o
      {
	filecfg(dati,strings);
	break;
      }
      case '4':				//salva configurazione su file
      {
	savecfg(dati,strings);
	break;
      }
      case '5':				//carica configurazione da file
      {
	clear();
	header(dati,strings);
	getmaxyx(stdscr, rows, cols);	//recupera la dimensione dello schermo
	if(rows<rowmin)
	  rows=rowmin;
	if(cols<colmin)
	  cols=colmin;
	roff=rows/2;			//centra in verticale
	attron(A_BOLD);
	mvprintw(roff-1,(cols-(strlen(strings->overwrite005)))/2,"%s",strings->overwrite005);	//This will overwrite your actual configuration.
	attroff(A_BOLD);
	//ARE YOU SURE YOU WANT TO PROCEED? (yes/no)
	mvprintw(roff+1,((cols-(strlen(strings->sure001)+6))/2),"%s (%c/%c)",strings->sure001,strings->yes,strings->no);curs_set(0);
	refresh();
	while(true)
	{
	  choice=getch();		//finche' l'utonto non preme y o n resta qui
	  if(choice==strings->no)
	    return;
	  else if(choice==strings->yes)
	    break;
	}
	readcfg(dati,strings);
	dati->serPort = open(dati->port, O_RDWR | O_NOCTTY | O_NDELAY);
	if (dati->serPort == -1)
	{
	  seropenerr(dati,strings);
	}
	else
	{
	  fcntl(dati->serPort, F_SETFL, 0);
	  initport(dati);
	}
	break;
      }
      case '9':				//uscita dal menu
      {
	clear();
	return;
	break;
      }
      default:
      {
	break;
      }
    }
  }
}


/********************************
 *           sercfg()           *
 ********************************
 Configura parametri porta seriale
 
Variabili esterne:
dati=struttura SYSDATA (vedere params.h)

Variabili interne:
coff=offset colonna
i=contatore generico
n=selettore porta seriale
rows,cols=geometria schermo
select=selezione funzione menu
*/
void sercfg(SYSDATA *dati,SYSTEXT *strings)
{
  int n=255,coff,i,select,rows,cols;
  while(true)
  {	
    clear();
    header(dati,strings);
    getmaxyx(stdscr, rows, cols);	//recupera la dimensione dello schermo
    if(rows<rowmin)
      rows=rowmin;
    if(cols<colmin)
      cols=colmin;
    attron(A_BOLD);
    mvprintw(6,(cols-(strlen(strings->menu030)))/2,"%s",strings->menu030);	//SERIAL PORT PARAMETERS
    attroff(A_BOLD);
    
    coff=(cols/2)-20;
    mvprintw(8,coff,"%s: %s",strings->str006,dati->port);	//Port
    mvprintw(9,coff,"%s: %i",strings->str007,dati->portcount);	//Ports found
    
    mvprintw(11,coff,"%s: %i",strings->str008,dati->speed);	//Speed

    attron(A_BOLD);
    mvprintw(12,3,"%s",strings->str009);	//NOTE
    attroff(A_BOLD);
    printw(": %s",strings->str010);		//: change speed only if you are sure, otherwise you will be
    mvprintw(13,21,"%s",strings->str011);	//unable to communicate with the MCU

    coff=(cols/2)-10;			//il menu' viene centrato in orizzontale, ma non in verticale
    mvprintw(16,coff,"1) %s",strings->menu031);	//Toggle port

    mvprintw(18,coff,"2) %s",strings->menu032);	//Toggle speed

    mvprintw(21,coff,"9) %s",strings->menu010);	//EXIT
    curs_set(0);
    refresh();
    select=getch();
    switch(select)
    {
      case '1':				//imposta porta successiva
      {
	if(n<((dati->portcount)-1))
	  n++;
	else
	  n=0;
	for(i=0;i<15;i++)
	  dati->port[i]=dati->ports[(n*maxport)+i];
	break;
      }
      case '2':				//imposta velocita' successiva
      {
	switch(dati->speed)
	{
	  case 300:
	  {
	    dati->speed=600;
	    break;
	  }
	  case 600:
	  {
	    dati->speed=1200;
	    break;
	  }
	  case 1200:
	  {
	    dati->speed=2400;
	    break;
	  }
	  case 2400:
	  {
	    dati->speed=4800;
	    break;
	  }
	  case 4800:
	  {
	    dati->speed=9600;
	    break;
	  }
	  case 9600:
	  {
	    dati->speed=19200;
	    break;
	  }
	  case 19200:
	  {
	    dati->speed=38400;
	    break;
	  }
	  case 38400:
	  {
	    dati->speed=57600;
	    break;
	  }
	  case 57600:
	  {
	    dati->speed=115200;
	    break;
	  }
	  case 115200:
	  {
	    dati->speed=300;
	    break;
	  }
	  default:
	  {
	    dati->speed=300;
	    break;
	  }
	}
	break;
      }
      case '9':				//uscita dal menu
      {
	clear();
	if(!fileexists(dati->port))	//verifica se la porta impostata esiste
	{
	  dati->serPort=-1;		//no, imposta valore porta a -1 (serve per header())
	  seropenerr(dati,strings);	//visualizza errore
	}
	return;
	break;
      }
      default:
      {
	break;
      }
    }
  }
}


/********************************
 *           memcfg()           *
 ********************************
 Configura parametri EEPROM
  
Variabili esterne:
dati=struttura SYSDATA (vedere params.h)

Variabili interne:
coff=offset colonna
rows,cols=geometria schermo
select=selezione funzione menu
prsize=dimensione eeprom precedente
cbuffer[],bbuffer[]=buffer dinamici gestione eeprom
*/
void memcfg(SYSDATA *dati,SYSTEXT *strings)
{
  int coff,select,rows,cols,prsize=dati->eesize;
  char *cbuffer;
  bool *bbuffer;
  while(true)
  {
    clear();
    header(dati,strings);
    getmaxyx(stdscr, rows, cols);	//recupera la dimensione dello schermo
    if(rows<rowmin)
      rows=rowmin;
    if(cols<colmin)
      cols=colmin;
    attron(A_BOLD);
    mvprintw(6,(cols-(strlen(strings->menu040)))/2,"%s",strings->menu040);	//EEPROM PARAMETERS
    attroff(A_BOLD);
    coff=(cols/2)-20;
    mvprintw(8,coff,"%s: %i",strings->str012,dati->eesize);	//EEPROM size
    mvprintw(9,coff,"%s: ",strings->str013);	//EEPROM Write mode
    if(dati->eemode=='m')
      printw("%s",strings->str014);	//Write only modified cells
    else
      printw("%s",strings->str015);	//Overwrite all cells
    mvprintw(10,coff,"%s: ",strings->str016);	//EEPROM Data mode
    if(dati->eedata=='n')
      printw("%s",strings->str017);	//Number
    else
      printw("%s",strings->str018);	//Character

    coff=(cols/2)-10;			//il menu' viene centrato in orizzontale, ma non in verticale
    mvprintw(12,coff,"1) %s",strings->menu041);	//Toggle EEPROM Size

    mvprintw(14,coff,"2) %s",strings->menu042);	//Toggle EEPROM Write mode

    mvprintw(16,coff,"3) %s",strings->menu043);	//Toggle EEPROM Data mode

    mvprintw(21,coff,"9) %s",strings->menu010);	//EXIT
    curs_set(0);
    refresh();
    select=getch();
    switch(select)
    {
      case '1':				//imposta dimensione eeprom successiva
      {
	switch(dati->eesize)
	{
	  case 64:
	  {
	    dati->eesize=128;
	    break;
	  }
	  case 128:
	  {
	    dati->eesize=256;
	    break;
	  }
	  case 256:
	  {
	    dati->eesize=512;
	    break;
	  }
	  case 512:
	  {
	    dati->eesize=1024;
	    break;
	  }
	  case 1024:
	  {
	    dati->eesize=64;
	    break;
	  }
	  default:
	  {
	    dati->eesize=64;
	    break;
	  }
	}
	break;
      }
      case '2':				//cambia modalita' scrittura eeprom
      {
	if(dati->eemode=='m')
	  dati->eemode='o';
	else
	  dati->eemode='m';
	break;
      }
      case '3':				//cambia modo visualizzazione eeprom
      {
	if(dati->eedata=='n')
	  dati->eedata='c';
	else
	  dati->eedata='n';
	break;
      }
      case '9':				//uscita dal menu
      {
	clear();
	cbuffer=(char*)calloc(sizeof(char),prsize+1);	//alloca buffer char
	bbuffer=(bool*)calloc(sizeof(bool),prsize+1);	//alloca buffer bool
	for(coff=0;coff<prsize;coff++)	//salva dati precedenti
	{
	  cbuffer[coff]=dati->ee[coff];
	  bbuffer[coff]=dati->eemod[coff];
	}
	if(dati->ee!=NULL)
	  free(dati->ee);
	if(dati->eemod!=NULL)
	  free(dati->eemod);
	dati->ee=(char*)calloc(sizeof(char),dati->eesize);	//rialloca la giusta dimensione
	dati->eemod=(bool*)calloc(sizeof(bool),dati->eesize);
	if(prsize>=dati->eesize)	//ripristina dati precedenti
	{
	  for(coff=0;coff<dati->eesize;coff++)
	  {
	    dati->ee[coff]=cbuffer[coff];
	    dati->eemod[coff]=bbuffer[coff];
	  }
	}
	else
	{
	  for(coff=0;coff<prsize;coff++)
	  {
	    dati->ee[coff]=cbuffer[coff];
	    dati->eemod[coff]=bbuffer[coff];
	  }
	  for(coff=prsize;coff<dati->eesize;coff++)
	  {
	    dati->ee[coff]=0;
	    dati->eemod[coff]=true;
	  }
	}
	if(cbuffer!=NULL)		//libera buffers
	  free(cbuffer);
	if(bbuffer!=NULL)
	  free(bbuffer);
	return;
	break;
      }
      default:
      {
	break;
      }
    }
  }
  return;
}


/*********************************
 *          filecfg()           *
 ********************************
 Configura file EEPROM
  
Variabili esterne:
dati=struttura SYSDATA (vedere params.h)

Variabili interne:
buffer[]=buffer dinamico nome file
rows,cols=geometria schermo
select=selezione funzione menu
*/
void filecfg(SYSDATA *dati,SYSTEXT *strings)
{
  int select,rows,cols;
  char *buffer;
  while(true)
  {
    clear();
    header(dati,strings);
    getmaxyx(stdscr, rows, cols);	//recupera la dimensione dello schermo
    if(rows<rowmin)
      rows=rowmin;
    if(cols<colmin)
      cols=colmin;
					//il menu' viene centrato in orizzontale, ma non in verticale
    attron(A_BOLD);
    mvprintw(6,(cols-(strlen(strings->menu050)))/2,"%s",strings->menu050);	//FILE I/O
    attroff(A_BOLD);
    mvprintw(8,(cols/2)-10,"%s:",strings->str019);	//Actual EEPROM file

    mvprintw(10,(cols-(strlen(dati->memfile)))/2,"%s",dati->memfile);

    mvprintw(13,(cols/2)-10,"1) %s",strings->menu051);	//Change EEPROM file

    mvprintw(21,(cols/2)-10,"9) %s",strings->menu010);	//EXIT
    curs_set(0);
    refresh();
    select=getch();
    switch(select)
    {
      case '1':				//cambia file eeprom
      {
	clear();
	header(dati,strings);
	getmaxyx(stdscr, rows, cols);	//recupera la dimensione dello schermo
	if(rows<rowmin)
	  rows=rowmin;
	if(cols<colmin)
	  cols=colmin;
	attron(A_BOLD);
	mvprintw(6,(cols-(strlen(strings->menu050)))/2,"%s",strings->menu050);	//FILE I/O
	attroff(A_BOLD);
	mvprintw(8,(cols/2)-10,"%s:",strings->str019);	//Actual EEPROM file

	mvprintw(10,(cols-strlen(dati->memfile))/2,"%s",dati->memfile);

	mvprintw(13,(cols/2)-10,"%s:",strings->str020);	//New EEPROM file

	mvprintw(15,0,">>>");
	refresh();
	curs_set(1);
	echo();
	buffer=(char*)calloc(sizeof(char),maxfile+1);
	mvgetnstr(15,4,buffer,maxfile);
	if(dati->memfile!=NULL)
	  free(dati->memfile);
	dati->memfile=(char*)calloc(sizeof(char),strlen(buffer)+1);
	strcpy(dati->memfile,buffer);
	if(buffer!=NULL)
	  free(buffer);
	noecho();
	curs_set(0);
	break;
      }
      case '9':				//uscita dal menu
      {
	clear();
	return;
	break;
      }
      default:
      {
	break;
      }
    }
  }
  return;
}


/********************************
 *          seropenerr()        *
 ********************************
 Visualizza errore apertura porta seriale
  
Variabili esterne:
dati=struttura SYSDATA (vedere params.h)

Variabili interne:
roff=offset riga
rows,cols=geometria schermo
*/
void seropenerr(SYSDATA *dati,SYSTEXT *strings)
{
   int roff,rows,cols;
   if(dati->cmdline)			//messaggio di errore in modalita' parametrica
   {
     printf("%s %s.\n\n",strings->err001,dati->port);//Unable to open port *
   }
   else					//messaggio di errore in modalita' grafica
   {
     clear();
     header(dati,strings);
     getmaxyx(stdscr, rows, cols);	//recupera la dimensione dello schermo
     if(rows<rowmin)
       rows=rowmin;
     if(cols<colmin)
       cols=colmin;
     roff=rows/2;				//centra in verticale
     attron(A_BOLD);
     mvprintw(roff-2,((cols-(strlen(strings->err001)))-strlen(dati->port))/2,"%s %s.",strings->err001,dati->port);//Unable to open port *
     attroff(A_BOLD);
     mvprintw(roff,((cols-(strlen(strings->setchk002)))/2),"%s",strings->setchk002);	//You should check your settings and hardware.
     mvprintw(roff+2,2,"%s",strings->strike001);	//Strike a key to continue...
     curs_set(0);
     refresh();
     getch();
   }
   return;
}


/********************************
 *           serioerr()         *
 ********************************
 Visualizza errore I/O porta seriale
 
Variabili esterne:
dati=struttura SYSDATA (vedere params.h)

Variabili interne:
roff=offset riga
rows,cols=geometria schermo
*/
void serioerr(SYSDATA *dati,SYSTEXT *strings)
{
   int roff,rows,cols;
   if(dati->cmdline)			//messaggio di errore in modalita' parametrica
   {
     printf("%s %s.\n\n",strings->err002,dati->port);//I/O error on port
   }
   else					//messaggio di errore in modalita' grafica
   {
     clear();
     header(dati,strings);
     getmaxyx(stdscr, rows, cols);	//recupera la dimensione dello schermo
     if(rows<rowmin)
       rows=rowmin;
     if(cols<colmin)
       cols=colmin;
     roff=rows/2;				//centra in verticale
     attron(A_BOLD);
     mvprintw(roff-2,((cols-19)-strlen(dati->port))/2,"%s %s.",strings->err002,dati->port);//I/O error on port
     attroff(A_BOLD);
     mvprintw(roff,((cols-(strlen(strings->setchk002)))/2),"%s",strings->setchk002);	//You should check your settings and hardware.
     mvprintw(roff+2,2,"%s",strings->strike001);	//Strike a key to continue...
     curs_set(0);
     refresh();
     getch();
   }
   return;
}


/********************************
 *           ioerror()          *
 ********************************
 Visualizza errore I/O

Variabili esterne:
dati=struttura SYSDATA (vedere params.h)
nome[]=nome file da visualizzare

Variabili interne:
roff=offset riga
rows,cols=geometria schermo
*/
void ioerror(SYSDATA *dati,SYSTEXT *strings,char *nome)
{
   int roff,rows,cols;
   if(dati->cmdline)			//messaggio di errore in modalita' parametrica
   {
     printf("%s\n",strings->err003);	//I/O error.
     printf("%s %s.\n\n",strings->err004,nome);//Cannot operate on file
   }
   else					//messaggio di errore in modalita' grafica
   {
     clear();
     header(dati,strings);
     getmaxyx(stdscr, rows, cols);	//recupera la dimensione dello schermo
     if(rows<rowmin)
       rows=rowmin;
     if(cols<colmin)
       cols=colmin;
     roff=rows/2;				//centra in verticale
     attron(A_BOLD);
     mvprintw(roff-2,((cols-(strlen(strings->err003)))/2),"%s",strings->err003);	//I/O error.
     attroff(A_BOLD);
     mvprintw(roff,((cols-(strlen(strings->err004)))-strlen(nome))/2,"%s %s.",strings->err004,nome);//Cannot operate on file
     mvprintw(roff+2,2,"%s",strings->strike001);	//Strike a key to continue...
     curs_set(0);
     refresh();
     getch();
   }
   return;
}


/********************************
 *            info()            *
 ********************************
 Info screen
 
Variabili esterne:
dati=struttura SYSDATA (vedere params.h)

Variabili interne:
i=contatore offset 1
ins=lettura input utente
j=contatore offset 2 (rimozione spazi ad inizio riga)
offset=offset visualizzazione vettore
rcount=contatore righe visualizzate
rows,cols=geometria schermo
scrrows=righe disponibili per la visualizzazione
text[]=testo da visualizzare
*/
void info(SYSDATA *dati,SYSTEXT *strings)
{
   int ins=0,i,j,n=0,scrrows,rcount,offset,rows,cols;

   offset=0;
   do
   {
     clear();
     header(dati,strings);
     getmaxyx(stdscr, rows, cols);	//recupera la dimensione dello schermo
     if(rows<rowmin)
       rows=rowmin;
     if(cols<colmin)
       cols=colmin;
     refresh();
     scrrows=(rows-headrows-4);		//calcola righe disponibili
     for(rcount=0,j=0;rcount<scrrows;rcount++)	//visualizza testo nel riquadro
     {
       while((strings->info001[j+offset])==' ')
	 j++;
       if((cols-4)+j+offset>=strlen(strings->info001))	//verifica primo spazio a cui andare a capo
	 n=0;				//la stringa finisce prima della riga
       else
       {
	 n=cols-4;			//fine riga
	 while(strings->info001[n+j+offset]!=' ')
	 {
	   n--;				//cerca primo spazio
	 }
	 n=cols-4-n;			//recupera offset
       }
       for(i=0;i<(cols-4-n)&&((strings->info001[i+j+offset])!='*');i++)
       {
	 if(i+j+offset>=strlen(strings->info001))	//fine stringa
	   break;
	 mvaddch(rcount+headrows+1,i+2,strings->info001[i+j+offset]);	//scrive carattere
       }
       j+=i;
       if((strings->info001[i+j+offset])=='*')	//carattere capo riga
	 j++;
     }
     mvaddch(headrows,0,ACS_ULCORNER);	//disegna riquadro
     for(i=0;i<(cols-2);i++)
       addch(ACS_HLINE);
     addch(ACS_URCORNER);
     for(i=(headrows+1);i<(rows-3);i++)
     {
       mvaddch(i,0,ACS_VLINE);
       mvaddch(i,(cols-1),ACS_VLINE);
     }
     mvaddch((rows-3),0,ACS_LLCORNER);
     for(i=0;i<(cols-2);i++)
       addch(ACS_HLINE);
     addch(ACS_LRCORNER);
     mvprintw((rows-2),2,"%s",strings->pgupdown001);	//Press END to exit, PGUP and PGDOWN to scroll.
     curs_set(0);
     refresh();
     ins=getch();
     if(ins==KEY_PPAGE)			//pag su
     {
	offset-=cols-4;
	if(offset<0)
	  offset=0;
     }
     else if(ins==KEY_NPAGE)		//pag giu
     {
       if((offset+i+j)<=(strlen(strings->info001)+(cols-4)))
	  offset+=cols-4;
     }
   }
   while(ins!=KEY_END);			//cicla finche' non si preme end
   return;
}


/********************************
 *            help()            *
 ********************************
Help testuale

Variabili esterne:
dati=struttura SYSDATA (vedere params.h)

Variabili interne:

*/
void help(SYSDATA *dati)
{
  printf("---- GENERAL ----\n");
  printf("  -?\t\tThis help\n");
  printf("  -v\t\tShow full version info\n\n");

  printf("\n---- READ OPTIONS ----\n");
  printf("  -r\t\tRead all MCU EEPROM and dump to stdout\n");
  printf("\t\t\tParameters needed: port, speed, size.\n\n");

//attualmente non puo' funzionare, occorre aggiungere firmware al micro
//  printf("  -rc\t\tRead MCU EEPROM cell and dump to stdout\n");
//  printf("\t\t\tParameters needed: port, speed, address.\n\n");

  printf("  -rf\t\tRead all MCU EEPROM and put result in file\n");
  printf("\t\t\tParameters needed: port, speed, size, file.\n\n");
  
  printf("\n---- WRITE OPTIONS ----\n");
  printf("  -wc\t\tWrite MCU EEPROM cell\n");
  printf("\t\t\tParameters needed: port, speed, address, data.\n\n");
  
  printf("  -wf\t\tWrite all MCU EEPROM from file\n");
  printf("\t\t\tParameters needed: port, speed, size, file.\n\n");

  printf("\n---- OTHER OPTIONS ----\n");
  printf("  -ping\t\tCommunication test\n");
  printf("\t\t\tParameters needed: port, speed.\n\n");
  
  printf("\n---- PARAMETERS ----\n");
  printf("  -p <port>\tSerial port to use. Full path is needed (like \"/dev/ttyS0\", not \"ttyS0\")\n");
  printf("  -s <size>\tMCU EEPROM size in bytes\n");
  printf("  -a <addr>\tCell address (decimal value)\n");
  printf("  -v <speed>\tSerial port speed in bps. Default should be 19200\n");
  printf("  -d <data>\tCell vaule (decimal value)\n");
  printf("  -f <file>\tEEPROM File to use. Full path is needed (like \"/tmp/eeprom.txt\", not \"eeprom.txt\")\n\n");
  
  printf("\n---- EXAMPLES ----\n");
  printf("  PIC-232 -r -p /dev/ttyS0 -v 19200 -s 256\n");
  printf("    Tries to read 256 bytes from ttyS0 at 19200bps \n\n");
  
  printf("  PIC-232 -rc -p /dev/ttyS0 -v 38400 -a 131\n");
  printf("    Tries to read byte 131 from ttyS0 at 38400bps \n\n");
  
  printf("  PIC-232 -wf -p /dev/ttyS0 -v 9600 -s 512 -f /tmp/eeprom.txt\n");
  printf("    Tries to write 512 bytes from /tmp/eeprom.txt to ttyS0 at 9600bps \n\n");
  
  printf("  PIC-232 -ping -p /dev/ttyS0 -v 19200\n");
  printf("    Communication test on port ttyS0 at 19200bps\n\n\n");
  
  return;
}

/********************************
 *         cnfscreen1()         *
 ********************************
Richiesta conferma modifica EEPROM locale
 
Variabili esterne:
dati=struttura SYSDATA (vedere params.h)

Variabili interne:
rows,cols=geometria schermo
roff=offset riga
choice=input utente
*/
int cnfscreen1(SYSDATA *dati,SYSTEXT *strings)
{
  int roff,rows,cols;
  char choice;
  clear();
  header(dati,strings);
  getmaxyx(stdscr, rows, cols);		//recupera la dimensione dello schermo
  if(rows<rowmin)
    rows=rowmin;
  if(cols<colmin)
    cols=colmin;
  roff=rows/2;				//centra in verticale
  //This will change the local copy of EEPROM data with the content
  mvprintw(roff-1,((cols-(strlen(strings->overwrite006)))/2),"%s",strings->overwrite006);
  mvprintw(roff,((cols-(strlen(strings->overwrite007)))/2),"%s",strings->overwrite007);//of the MCU EEPROM.
  //ARE YOU SURE YOU WANT TO PROCEED? (yes/no)
  mvprintw(roff+2,((cols-(strlen(strings->sure001)+6))/2),"%s (%c/%c)",strings->sure001,strings->yes,strings->no);curs_set(0);
  refresh();
  while(true)
  {
    choice=getch();			//finche' l'utonto non preme y o n resta qui
    if(choice==strings->no)
      return 1;
    else if(choice==strings->yes)
      return 0;
  }
}

/********************************
 *         cnfscreen2()         *
 ********************************
Richiesta conferma modifica EEPROM remota
 
Variabili esterne:
dati=struttura SYSDATA (vedere params.h)

Variabili interne:
rows,cols=geometria schermo
roff=offset riga
choice=input utente
*/
int cnfscreen2(SYSDATA *dati,SYSTEXT *strings)
{
  int roff,rows,cols;
  char choice;
  clear();
  header(dati,strings);
  getmaxyx(stdscr, rows, cols);		//recupera la dimensione dello schermo
  if(rows<rowmin)
    rows=rowmin;
  if(cols<colmin)
    cols=colmin;
  roff=rows/2;				//centra in verticale
  //This will overwrite the content of the MCU EEPROM.
  mvprintw(roff-1,((cols-(strlen(strings->overwrite008)))/2),"%s",strings->overwrite008);
  //ARE YOU SURE YOU WANT TO PROCEED? (yes/no)
  mvprintw(roff+1,((cols-(strlen(strings->sure001)+6))/2),"%s (%c/%c)",strings->sure001,strings->yes,strings->no);curs_set(0);
  refresh();
  while(true)
  {
    choice=getch();			//finche' l'utonto non preme y o n resta qui
    if(choice==strings->no)
      return 1;
    else if(choice==strings->yes)
      return 0;
  }
}

/********************************
 *         cnfscreen3()         *
 ********************************
Ping eseguito
 
Variabili esterne:
dati=struttura SYSDATA (vedere params.h)

Variabili interne:
rows,cols=geometria schermo
roff=offset riga
*/
void cnfscreen3(SYSDATA *dati,SYSTEXT *strings)
{
  int roff,rows,cols;
  clear();
  header(dati,strings);
  getmaxyx(stdscr, rows, cols);		//recupera la dimensione dello schermo
  if(rows<rowmin)
    rows=rowmin;
  if(cols<colmin)
    cols=colmin;
  roff=rows/2;				//centra in verticale
  mvprintw(roff-1,((cols-(strlen(strings->success006)))/2),"%s",strings->success006);//Ping test executed successfully.
  mvprintw(roff+1,2,"%s",strings->strike001);	//Strike a key to continue...
  curs_set(0);
  refresh();
  getch();
  return;
}
	

/********************************
 *        eediffscreen1()       *
 ********************************
Avviso dimensioni EEPROM locali maggiori di quelle remote
 
Variabili esterne:
dati=struttura SYSDATA (vedere params.h)

Variabili interne:
rows,cols=geometria schermo
roff=offset riga
*/
void eediffscreen1(SYSDATA *dati,SYSTEXT *strings)
{
  int roff,rows,cols;
  clear();
  header(dati,strings);
  getmaxyx(stdscr, rows, cols);		//recupera la dimensione dello schermo
  if(rows<rowmin)
    rows=rowmin;
  if(cols<colmin)
    cols=colmin;
  roff=rows/2;				//centra in verticale
  //EEPROM size is set to * bytes, but the MCU reported less.
  mvprintw(roff-2,((cols-(strlen(strings->eesize001)+strlen(strings->eesize006)+12))/2),"%s %i bytes, %s",strings->eesize001,dati->eesize,strings->eesize006);
  mvprintw(roff,((cols-(strlen(strings->setchk001)))/2),"%s",strings->setchk001);//You should check your settings.
  mvprintw(roff+2,2,"%s",strings->strike001);	//Strike a key to continue...
  curs_set(0);
  refresh();
  getch();
  return;
}

/********************************
 *        eediffscreen2()       *
 ********************************
Avviso dimensioni EEPROM remote maggiori di quelle locali
 
Variabili esterne:
dati=struttura SYSDATA (vedere params.h)

Variabili interne:
rows,cols=geometria schermo
roff=offset riga
*/
void eediffscreen2(SYSDATA *dati,SYSTEXT *strings)
{
  int roff,rows,cols;
  clear();
  header(dati,strings);
  getmaxyx(stdscr, rows, cols);		//recupera la dimensione dello schermo
  if(rows<rowmin)
    rows=rowmin;
  if(cols<colmin)
    cols=colmin;
  roff=rows/2;				//centra in verticale
  //EEPROM size is set to * bytes, but the MCU reported more.
  mvprintw(roff-2,((cols-(strlen(strings->eesize001)+strlen(strings->eesize006)+12))/2),"%s %i bytes, %s",strings->eesize001,dati->eesize,strings->eesize006);
  mvprintw(roff,((cols-(strlen(strings->setchk001)))/2),"%s",strings->setchk001);//You should check your settings.
  mvprintw(roff+2,2,"%s",strings->strike001);	//Strike a key to continue...
  curs_set(0);
  refresh();
  getch();
  return;
}

/********************************
 *        eediffscreen3()       *
 ********************************
Avviso EEPROM non modificata
 
Variabili esterne:
dati=struttura SYSDATA (vedere params.h)

Variabili interne:
rows,cols=geometria schermo
roff=offset riga
*/
void eediffscreen3(SYSDATA *dati,SYSTEXT *strings)
{
  int roff,rows,cols;
  clear();
  header(dati,strings);
  getmaxyx(stdscr, rows, cols);		//recupera la dimensione dello schermo
  if(rows<rowmin)
    rows=rowmin;
  if(cols<colmin)
    cols=colmin;
  roff=rows/2;				//centra in verticale
  mvprintw(roff-2,((cols-(strlen(strings->success004)))/2),"%s",strings->success004);//MCU EEPROM seems to be already updated.
  mvprintw(roff,((cols-(strlen(strings->success005)))/2),"%s",strings->success005);	//No data has been written.
  mvprintw(roff+2,2,"%s",strings->strike001);	//Strike a key to continue...
  curs_set(0);
  refresh();
  getch();
  return; 				//fine comunicazione
}

/********************************
 *        eediffscreen4()       *
 ********************************
Avviso errore scrittura EEPROM
 
Variabili esterne:
dati=struttura SYSDATA (vedere params.h)
n=cella difettosa

Variabili interne:
rows,cols=geometria schermo
roff=offset riga
*/
void eediffscreen4(SYSDATA *dati,SYSTEXT *strings,int n)
{
  int roff,rows,cols;
  clear();
  header(dati,strings);
  getmaxyx(stdscr, rows, cols);		//recupera la dimensione dello schermo
  if(rows<rowmin)
    rows=rowmin;
  if(cols<colmin)
    cols=colmin;
  roff=rows/2;				//centra in verticale
  //System board reported an error writing on EEPROM cell *.
  mvprintw(roff-2,((cols-(strlen(strings->err005)))/2),"%s %i.",strings->err005,(n+1));
  mvprintw(roff,((cols-(strlen(strings->err006)))/2),"%s",strings->err006);//Write process will be terminated.
  mvprintw(roff+2,2,"%s",strings->strike001);	//Strike a key to continue...
  curs_set(0);
  refresh();
  getch();
  return; 				//fine comunicazione
}

/********************************
 *          graphbar()          *
 ********************************
Barra grafica avanzamento lettura/scrittura
 
Variabili esterne:
dati=struttura SYSDATA (vedere params.h)
barra=avanzamento barra
n=numero bytes eseguiti

Variabili interne:
rows,cols=geometria schermo
i=contatore generico
*/
void graphbar(SYSDATA *dati,SYSTEXT *strings,bool *barra,int n)
{
  int rows,cols,i;
  clear();
  header(dati,strings);
  getmaxyx(stdscr, rows, cols);	//recupera la dimensione dello schermo
  if(rows<rowmin)
    rows=rowmin;
  if(cols<colmin)
    cols=colmin;
  mvprintw(8,((cols-72)/2),"0 [");	//visualizza barra avanzamento
  mvprintw(8,((cols-72)/2)+67,"] 100");
  mvprintw(10,((cols-(14+(strlen(strings->str003))))/2),"byte %i %s %i",(n+1),strings->str003,dati->eesize);//byte * of *
  for (i=0;i<64&&barra[i];i++)
  {
    if(barra[i])
      mvaddch(8,i+7,ACS_CKBOARD);
  }
  curs_set(0);
  refresh();
}

/********************************
 *        graphbarmove()        *
 ********************************
Aggiornamento barra grafica
 
Variabili esterne:
dati=struttura SYSDATA (vedere params.h)
barra=avanzamento barra
n=numero bytes eseguiti

Variabili interne:
rows,cols=geometria schermo
i=contatore generico
*/
void graphbarmove(SYSDATA *dati,SYSTEXT *strings,bool *barra,int n)
{
  int rows,cols,i;
  getmaxyx(stdscr, rows, cols);	//recupera la dimensione dello schermo
  if(rows<rowmin)
    rows=rowmin;
  if(cols<colmin)
    cols=colmin;
  mvprintw(10,((cols-(14+(strlen(strings->str003))))/2),"byte %i %s %i",(n+1),strings->str003,dati->eesize);//byte * of *
  for (i=0;i<64&&barra[i];i++)
  {
    if(barra[i])
    {
      mvaddch(8,((cols-72)/2)+3+i,ACS_CKBOARD);
    }
  }
  curs_set(0);
  refresh();
  return;
}
