/*****************************************************************************
 *                                                                           *
 * PIC-232 - Comunicazione seriale per gestione memoria EEPROM 877 System    *
 * Copyright (C) 2009  by Walter Lain <kcswalter@member.fsf.org>             *
 *                                                                           *
 * This program is free software; you can redistribute it and/or modify      *
 * it under the terms of the GNU General Public License as published by      *
 * the Free Software Foundation; either version 3 of the License, or         *
 * (at your option) any later version.                                       *
 *                                                                           *
 * This program is distributed in the hope that it will be useful,           *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 * GNU General Public License for more details.                              *
 *                                                                           *
 * You should have received a copy of the GNU General Public License         *
 * along with this program; if not, write to the Free Software               *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
 *                                                                           *
 *****************************************************************************/


/********************************
 *            LIBRERIE          *
 ********************************/

#include <stdio.h>   			// Standard input/output definitions
#include <stdlib.h>	  		//
#include <string.h>  			// String function definitions
#include <unistd.h>  			// UNIX standard function definitions
#include <fcntl.h>   			// File control definitions
#include <errno.h>   			// Error number definitions
#include <termios.h> 			// POSIX terminal control definitions
#include <math.h>			// matematica per "ceil" e "pow"
#include <ncurses.h>			// terminal graphics
#include <signal.h>			// segnali di sistema
#include <sys/types.h>			//
#include <sys/stat.h>			//
#include <unistd.h>			//


/********************************
 *          DEFINIZIONI         *
 ********************************/

typedef struct sysdata SYSDATA;		//struttura dati condivisi

struct sysdata{
  char *ee;				//mappa memoria EEPROM
  char eedata;				//modalita' gestione memoria (n=integer,c=character)
  bool *eemod;				//mappa celle EEPROM modificate
  char eemode;				//modalita' scrittura memoria (m=modifica,o=overwrite)
  int eesize;				//dimensioni memoria EEPROM
  char *memfile;			//nome file memoria
  char *port;				//indirizzo porta seriale in uso
  int portcount;			//numero porte seriali trovate
  char *ports;				//elenco porte seriali trovate
  int serPort;				//oggetto porta seriale
  int speed;				//velocita' porta seriale
  bool cmdline;				//gestione argomenti da linea di comando
};

typedef struct localization SYSTEXT;	//struttura messaggi localizzati

struct localization{
  char yes;
  char no;
  char *eesize001;
  char *eesize002;
  char *eesize003;
  char *eesize004;
  char *eesize005;
  char *eesize006;
  char *eesize007;
  char *err001;
  char *err002;
  char *err003;
  char *err004;
  char *err005;
  char *err006;
  char *header001;
  char *header002;
  char *header003;
  char *header004;
  char *header005;
  char *header006;
  char *header007;
  char *info001;
  char *menu001;
  char *menu002;
  char *menu003;
  char *menu004;
  char *menu005;
  char *menu006;
  char *menu007;
  char *menu008;
  char *menu009;
  char *menu010;
  char *menu020;
  char *menu021;
  char *menu022;
  char *menu023;
  char *menu024;
  char *menu025;
  char *menu030;
  char *menu031;
  char *menu032;
  char *menu040;
  char *menu041;
  char *menu042;
  char *menu043;
  char *menu050;
  char *menu051;
  char *overwrite001;
  char *overwrite002;
  char *overwrite003;
  char *overwrite004;
  char *overwrite005;
  char *overwrite006;
  char *overwrite007;
  char *overwrite008;
  char *pgupdown001;
  char *setchk001;
  char *setchk002;
  char *splash001;
  char *str001;
  char *str002;
  char *str003;
  char *str004;
  char *str005;
  char *str006;
  char *str007;
  char *str008;
  char *str009;
  char *str010;
  char *str011;
  char *str012;
  char *str013;
  char *str014;
  char *str015;
  char *str016;
  char *str017;
  char *str018;
  char *str019;
  char *str020;
  char *strike001;
  char *success001;
  char *success002;
  char *success003;
  char *success004;
  char *success005;
  char *success006;
  char *sure001;
};

//---- bytes comando seriale (vedere protocollo)
#define ser_cmdread   169 //10101001
#define ser_cmdwrite   85 //01010101
#define ser_cmdwrext   86 //01010110
#define ser_cmdping   173 //10101101

//---- bytes ack seriale (vedere protocollo)
#define ser_ackread   131 //10000011
#define ser_ackwrite  195 //11000011
#define ser_acknowrt  196 //11000100
#define ser_ackwrext  197 //11000101
#define ser_acknoext  198 //11000110
#define ser_ackping     3 //00000011
#define ser_ackend    170 //10101010
#define ser_ackindok   27 //00011011
#define ser_ackinder   43 //00101011
#define ser_ackdtok    51 //00110011	
#define ser_ackdter    83 //01010011
#define ser_ackblker   91 //01011011 int argc, char *argv[] 
#define ser_ackrcver   87 //01010111
#define ser_ackwrtok  103 //01100111
#define ser_ackwrter  167 //10100111

//---- parametri sistema
#define cfgfile "config.txt"	//file configurazione
#define locfile "strings.txt"	//file localizzazione
#define maxmem  1024		//massima dimensione EEPROM gestita
#define maxfile  256		//massima dimensione nome file EEPROM
#define maxport   14		//massima dimensione nome porta seriale
#define rowmin 	  24		//dimensione minima righe
#define colmin    80		//dimensione minima colonne
#define headrows   4		//righe occupate dall'header

//---- revisione software
#define rev1 1
#define rev2 2
#define rev3 'c'
#define year 2011
#define date 109

//---- DEBUG
//#define DEBUG true
#define DEBUG false

/********************************
 *      DICHIARAZIONE SUB       *
 ********************************/

//---- MENU' e grafica
//splash screen iniziale
void splash(bool cmdline,SYSTEXT *strings);
//testata del programma
void header(SYSDATA *dati,SYSTEXT *strings);
//menu principale
int menu(SYSDATA *dati,SYSTEXT *strings);
//menu configurazione
void setprog(SYSDATA *dati,SYSTEXT *strings);
//impostazioni seriale
void sercfg(SYSDATA *dati,SYSTEXT *strings);
//impostazioni EEPROM
void memcfg(SYSDATA *dati,SYSTEXT *strings);
//impostazioni file
void filecfg(SYSDATA *dati,SYSTEXT *strings);
//schermata help grafico
void info(SYSDATA *dati,SYSTEXT *strings);
//schermata help testuale
void help(SYSDATA *dati);
//errore apertura seriale
void seropenerr(SYSDATA *dati,SYSTEXT *strings);
//errore comunicazione
void serioerr(SYSDATA *dati,SYSTEXT *strings);
//errore i/o file
void ioerror(SYSDATA *dati,SYSTEXT *strings,char *nome);
//richiesta conferma modifica EEPROM locale
int cnfscreen1(SYSDATA *dati,SYSTEXT *strings);
//richiesta conferma modifica EEPROM remota
int cnfscreen2(SYSDATA *dati,SYSTEXT *strings);
//ping eseguito
void cnfscreen3(SYSDATA *dati,SYSTEXT *strings);
//avviso dimensioni EEPROM locali maggiori di quelle remote
void eediffscreen1(SYSDATA *dati,SYSTEXT *strings);
//avviso dimensioni EEPROM remote maggiori di quelle locali
void eediffscreen2(SYSDATA *dati,SYSTEXT *strings);
//avviso EEPROM non modificata
void eediffscreen3(SYSDATA *dati,SYSTEXT *strings);
//avviso errore scrittura EEPROM
void eediffscreen4(SYSDATA *dati,SYSTEXT *strings,int n);
//barra grafica avanzamento lettura/scrittura
void graphbar(SYSDATA *dati,SYSTEXT *strings,bool *barra,int n);
//aggiornamento barra grafica
void graphbarmove(SYSDATA *dati,SYSTEXT *strings,bool *barra,int n);

//---- GESTIONE DATI
//visualizza EEPROM byte
void e2shownum(SYSDATA *dati,SYSTEXT *strings);
//visualizza EEPROM ascii
void e2showchar(SYSDATA *dati,SYSTEXT *strings);
//modifica EEPROM byte
void e2modnum(SYSDATA *dati,SYSTEXT *strings);
//modifica EEPROM ascii
void e2modchar(SYSDATA *dati,SYSTEXT *strings);

//---- FILE
//lettura file EEPROM
int e2load(SYSDATA *dati,SYSTEXT *strings);
//scrittura file EEPROM
void e2save(SYSDATA *dati,SYSTEXT *strings);
//lettura file configurazione
int readcfg(SYSDATA *dati,SYSTEXT *strings);
//scrittura file configurazione
int savecfg(SYSDATA *dati,SYSTEXT *strings);
//caricamento localizzazione
int readlocal(bool cmdline,SYSTEXT *strings);

//---- I/O
//scrittura seriale
int serwrite(SYSDATA *dati,SYSTEXT *strings,char *dato,int);
//lettura seriale
int serread(SYSDATA *dati,char *dato);
//scrittura EEPROM
int e2write(SYSDATA *dati,SYSTEXT *strings);
//lettura EEPROM
int e2read(SYSDATA *dati,SYSTEXT *strings,bool showdata);
//init porta seriale
int initport(SYSDATA *dati);
//verifica comunicazione seriale
int ping(SYSDATA *dati,SYSTEXT *strings);
//enumerazione porte seriali
int serenum(SYSDATA *dati);

//---- SISTEMA
//coso per enumerazione seriali
bool fileexists(char *nome);
//chiusura ncurses
void chiudi(int sig);
