/*****************************************************************************
 *                                                                           *
 * PIC-232 - Comunicazione seriale per gestione memoria EEPROM 877 System    *
 * Copyright (C) 2009  by Walter Lain <kcswalter@member.fsf.org>             *
 *                                                                           *
 * This program is free software; you can redistribute it and/or modify      *
 * it under the terms of the GNU General Public License as published by      *
 * the Free Software Foundation; either version 3 of the License, or         *
 * (at your option) any later version.                                       *
 *                                                                           *
 * This program is distributed in the hope that it will be useful,           *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 * GNU General Public License for more details.                              *
 *                                                                           *
 * You should have received a copy of the GNU General Public License         *
 * along with this program; if not, write to the Free Software               *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
 *                                                                           *
 *****************************************************************************/

#include "params.h"			//definizioni programma


/********************************
 *           e2read()           *
 ********************************
Legge il contenuto della EEPROM

Variabili esterne:
dati=struttura SYSDATA (vedere params.h)
showdata=visualizza su stdout i dati letti (in modalita' parametrica)

Variabili interne:
barra[]=vettore barra avanzamento lettura
buffer=variabile temporanea per gestione caratteri seriali
c=contatore timeout lettura
ch=primo carattere seriale
err=contatore errori comunicazione
i=contatore ripetizioni errori
ibuff[]=buffer lettura seriale
itemp=buffer temporaneo carattere lettura seriale
n=contatore posizione memoria
obuff[]=buffer scrittura seriale
stat=stato sequenza lettura
z=contatore generico

ritorno:
0=tutto ok
1=errore apertura porta seriale
2=errore I/O porta seriale
3=dimensioni EEPROM non congrue
9=comando annullato dall'utente
*/
int e2read(SYSDATA *dati,SYSTEXT *strings,bool showdata)
{
   int c,stat,err,ch,buffer=0,z;
   unsigned int i=0,n=0;
   bool barra[65];
   char ibuff[10],obuff[1],itemp=0;
   if(!dati->cmdline)			//modalita' grafica, chiede conferma
     if(cnfscreen1(dati,strings)==1)	//avviso e richiesta di procedere
        return 9;			//risposta negativa
   dati->serPort = open(dati->port, O_RDWR | O_NOCTTY | O_NDELAY);	//prova ad aprire porta
   if (dati->serPort == -1)
   {
     seropenerr(dati,strings);		//errore apertura porta
     return 1;
   }
   else
   {
     fcntl(dati->serPort, F_SETFL, FNDELAY);	//configura porta in ricezione per ritornare subito (vedere guida posix)
     initport(dati);			//set velocita' comunicazione
   }
   for (i=0;i<65;i++)			//azzera barra avanzamento
   {
      barra[i]=false;
   }
   if(dati->cmdline)			//visualizza avanzamento testuale
   {
     printf("\tbyte %i %s %i\n",(n+1),strings->str003,dati->eesize);//byte * of *
   }
   else					//visualizza barra grafica
   {
     graphbar(dati,strings,barra,n);     
   }
   obuff[0]=ser_cmdread;		//pulisce buffer seriale
   if(DEBUG)				//visualizza output di debug
   {
     if(dati->cmdline)
     {
       printf("PC -> CMD_READ\n");
     }
     else
     {
       printw("PC -> CMD_READ\n");
       refresh();
     }
   }
   if(serwrite(dati,strings,obuff,1)==0)//se ci sono problemi di I/O si ferma
   {
     close(dati->serPort);		//chiude porta seriale
     serioerr(dati,strings);
     return 2;
   }
   i=0,err=0;
   do
   {
      for(stat=0,c=0;(stat<2&&n<(dati->eesize)-1)||(stat<1&&n==(dati->eesize)-1);c++)	//attesa dato
      {
	usleep(10);
	stat=serread(dati,ibuff); 	//legge dato
	ch=ibuff[0]&0x000000FF;
	if(stat==1&&ch==ser_ackend)	//se riceve fine trasmissione, lo gestisce anche qui
	{				//altrimenti dovrebbe attendere il secondo carattere
	  err=0;
	  c=0;
	  stat=2;
	  if(DEBUG)			//visualizza output di debug
	  {
	    if(dati->cmdline)
	    {
	      printf("ACK_END <- MCU\n");
	    }
	    else
	    {
	      printw("ACK_END <- MCU\n");
	      refresh();
	    }
	  }
	}
	else if(stat==1&&ch==ser_ackrcver)	//se riceve errore ricezione, lo gestisce anche qui
	{				//altrimenti dovrebbe attendere il secondo carattere
	  err=0;
	  c=0;
	  stat=2;
	  if(DEBUG)			//visualizza output di debug
	  {
	    if(dati->cmdline)
	    {
	      printf("ACK_RCVER <- MCU\n");
	    }
	    else
	    {
	      printw("ACK_RCVER <- MCU\n");
	      refresh();
	    }
	  }
	}
	else if(stat==1&&n<(dati->eesize)-1&&err==0)//Deve leggere due bytes (tranne che nell'ultima trasmissione), ma 
	{					//     la sfiga capita spesso. Se becca il momento in cui c'e' un solo
	  err=1;				//     byte, lo recupera e sistema le cose
	  itemp=ibuff[0];
	}
	else if(stat==1&&n<(dati->eesize)-1&&err==1)
	{
	  err=0;
	  ibuff[1]=ibuff[0];
	  ibuff[0]=itemp;
	  c=0;
	  stat=2;
	}
	if(c==5000)			//5000*10us = 0,5 sec di attesa dato
	{
	  close(dati->serPort);		//chiude porta seriale
	  serioerr(dati,strings);	//timeout
	  return 2;
	}
      }
      ch=ibuff[0]&0x000000FF;
      if((n==0&&ch==ser_ackread)||(n>0&&ch==ser_ackdter))	//comando ricevuto, legge dato
      {
	i=0;
	buffer=ibuff[1]&0x000000FF;
	obuff[0]=buffer;
	if(DEBUG)			//visualizza output di debug
	{
	  if(dati->cmdline)
	  {
	    if(ch==ser_ackread)
	      printf("ACK_READ <- MCU\n");
	    else
	      printf("ACK_DTER <- MCU\n");
	    printf("DATA (%i) <- MCU\n",buffer);
	    printf("PC -> DATA (%i)\n",obuff[0]);
	  }
	  else
	  {
	    if(ch==ser_ackread)
	      printw("ACK_READ <- MCU\n");
	    else
	      printw("ACK_DTER <- MCU\n");
	    printw("DATA (%i) <- MCU\n",buffer);
	    printw("PC -> DATA (%i)\n",obuff[0]);
	    refresh();
	  }
	}
	if(serwrite(dati,strings,obuff,1)==0)	//se ci sono problemi di I/O si ferma
	{
	  close(dati->serPort);		//chiude porta seriale
	  serioerr(dati,strings);
	  return 2;
	}
      }
      else if(ch==ser_ackdtok||ch==ser_ackend)	//dato ok, salva
      {
	if(DEBUG)			//visualizza output di debug
	{
	  if(dati->cmdline)
	  {
	    if(ch==ser_ackdtok)
	    {
	      printf("ACK_DTOK <- MCU\n");
	      printf("DATA (%i) <- MCU\n",buffer);
	    }
	    else
	      printf("ACK_END <- MCU\n");
	    
	  }
	  else
	  {
	    if(ch==ser_ackread)
	      printw("ACK_READ <- MCU\n");
	    else
	      printw("ACK_DTER <- MCU\n");
	    printw("DATA (%i) <- MCU\n",ibuff[1]);
	    refresh();
	  }
	}
	if(!showdata)
	{
	  if(buffer<255)
	    dati->ee[n]=buffer; 	//salva il dato in memoria
	  else
	    dati->ee[n]=255;
	  dati->eemod[n]=false;		//clear flag di modifica
	}
	else
	{
	  printf("> %i <\n",buffer);
	}
	if(ch==ser_ackend&&n<((dati->eesize)-1))//ha cercato di leggere piu' bytes di quelli disponibili
	{
	  close(dati->serPort);		//chiude porta seriale
	  if(!showdata)
	  {
	    for(z=n;z<(dati->eesize);z++)	//pulisce il resto del vettore
	      dati->ee[z]=255;
	  }
	  if(dati->cmdline)		//visualizza messaggio di errore in modalita' parametrica
	  {
	    printf("%s %i bytes, %s %i.\n",strings->eesize001,dati->eesize,strings->eesize005,n+1);//EEPROM size is set to * bytes, but the MCU reported only *.
	  }
	  else				//visualizza messaggio di errore in modalita' grafica
	  {
	    eediffscreen1(dati,strings);
	  }
	  return 3;
	}
	else if(n<(dati->eesize-1))	//controlla prossimo byte
	{
	  n++;
	  buffer=ibuff[1]&0x000000FF;
	  obuff[0]=buffer;
	  if(DEBUG)			//visualizza output di debug
	  {
	    if(dati->cmdline)
	    {
	      printf("PC -> DATA (%i)\n",obuff[0]);
	    }
	    else
	    {
	      printw("PC -> DATA (%i)\n",obuff[0]);
	      refresh();
	    }
	  }
	  if(serwrite(dati,strings,obuff,1)==0)	//se ci sono problemi di I/O si ferma
	  {
	    close(dati->serPort);	//chiude porta seriale
	    serioerr(dati,strings);
	    return 2;
	  }
	  i=0; 				//reset contatore timeout
	  barra[(int)(n/((dati->eesize)/64))]=true;	//aggiorna posizione barra
	  if(dati->cmdline)		//visualizza avanzamento testuale
	  {
	    printf("\tbyte %i %s %i\n",(n+1),strings->str003,dati->eesize);//byte * of *
	  }
	  else				//visualizza barra grafica
	  {
	    graphbarmove(dati,strings,barra,n);
	  }
	}
	else if(n==(dati->eesize)-1&&ch==ser_ackend)	//ultimo byte, non ne deve ricevere altri
	{
	  close(dati->serPort);		//chiude porta seriale
	  barra[(int)(n/((dati->eesize)/64))]=true;	//aggiorna posizione barra
	  if(dati->cmdline)		//visualizza avanzamento testuale
	  {
	    printf("\tbyte %i %s %i\n",(n+1),strings->str003,dati->eesize);//byte * of *
	  }
	  else				//visualizza barra grafica
	  {
	    graphbarmove(dati,strings,barra,n);
	    getch();
	  }
	  return 0; 			//fine comunicazione
	}
	else if(n==(dati->eesize)-1)	//il micro ha piu' bytes di quelli impostati
	{
	  close(dati->serPort);		//chiude porta seriale
	  if(dati->cmdline)		//visualizza messaggio di errore in modalita' parametrica
	  {
	    printf("%s %i bytes, %s\n",strings->eesize001,dati->eesize,strings->eesize006);
	  }
	  else				//visualizza messaggio di errore in modalita' grafica
	  {
	    eediffscreen2(dati,strings);
	  }
	  return 3;
	}
      }
      else if(ch==ser_ackrcver)		//errore ricezione seriale
      {
	close(dati->serPort);		//chiude porta seriale
	serioerr(dati,strings);
	return 2;
      }
      else
      {
	i++;
      }
   }
   while(i<10);
   close(dati->serPort);		//chiude porta seriale
   serioerr(dati,strings);		//se arriva qui ci sono stati troppi errori di comunicazione
   return 2;
}


/********************************
 *           e2write()          *
 ********************************
 Scrive il contenuto sulla EEPROM

Variabili esterne:
dati=struttura SYSDATA (vedere params.h)

Variabili interne:
barra[]=vettore barra avanzamento lettura
c=contatore errore timeout scrittura
ch=primo carattere seriale
cj=secondo carattere seriale
err=contatore errori comunicazione
i=contatore ripetizioni errori
ibuff[]=buffer lettura seriale
itemp=buffer temporaneo carattere lettura seriale
m=MSbyte indirizzo esteso
n=contatore posizione memoria
obuff[]=buffer scrittura seriale
stat=stato sequenza scrittura
z=contatore generico

ritorno:
0=tutto ok
1=errore apertura porta seriale
2=errore I/O porta seriale
3=dimensioni EEPROM non congrue
4=EEPROM non modificata
5=eeeore scrittura EEPROM
9=comando annullato dall'utente
*/
int e2write(SYSDATA *dati,SYSTEXT *strings)
{
   int c,z,stat,err,ch,cj,ctrl=0;
   unsigned int i=0,n=0,m=0;
   char ibuff[10],obuff[2],itemp=0;
   bool barra[65];
   if(!dati->cmdline)			//modalita' grafica, chiede conferma
     if(cnfscreen2(dati,strings)==1)	//avviso e richiesta di procedere
        return 9;			//risposta negativa
   for (i=0;i<65;i++)			//azzera barra avanzamento
   {
     barra[i]=false;
   }
   for(n=0;n<dati->eesize;n++)		//sistema i dati, 0-255
   {
     if((dati->ee[n])>255)
       dati->ee[n]=255;
     else if((dati->ee[n])<0)
       dati->ee[n]=0;
   }
   n=0;
   while(!(dati->eemod[n])&&n<(dati->eesize)&&(dati->eemode)=='m')	//se modalita' scrittura modifica
   {
     n++;				//carica primo indirizzo modificato
   }
   if(n==dati->eesize)			//non deve modificare niente
   {
      if(!dati->cmdline)		//modalita' grafica, visualizza messaggio
      {
	eediffscreen3(dati,strings);
	return 4;
      }
   }
   dati->serPort = open(dati->port, O_RDWR | O_NOCTTY | O_NDELAY);	//prova ad aprire porta
   if (dati->serPort == -1)
   {
     seropenerr(dati,strings);		//errore apertura porta
     return 1;
   }
   else
   {
     fcntl(dati->serPort, F_SETFL, FNDELAY);	//configura porta in ricezione per ritornare subito (vedere guida posix)
     initport(dati);			//set velocita' comunicazione
   }
   for (i=0;i<65;i++)			//aggiorna barra avanzamento
   {
      if(i<(n/((dati->eesize)/64)))
	barra[i]=true;
      else
	barra[i]=false;
   }
   if(dati->cmdline)			//visualizza avanzamento testuale
   {
     printf("\tbyte %i %s %i\n",(n+1),strings->str003,dati->eesize);//byte * of *
   }
   else					//visualizza barra grafica
   {
     graphbar(dati,strings,barra,n);
   }
   if((dati->eesize)>256)
   {
     obuff[0]=ser_cmdwrext;		//scrittura con due bytes di indirizzo
     if(DEBUG)				//visualizza output di debug
     {
       if(dati->cmdline)
       {
	 printf("PC -> CMD_WREXT\n");
       }
       else
       {
	 printw("PC -> CMD_WREXT\n");
	 refresh();
       }
     }
   }
   else
   {
     obuff[0]=ser_cmdwrite;		//scrittura con un byte di indirizzo
     if(DEBUG)				//visualizza output di debug
     {
       if(dati->cmdline)
       {
	 printf("PC -> CMD_WRITE\n");
       }
       else
       {
	 printw("PC -> CMD_WRITE\n");
	 refresh();
       }
     }
   }
   if(serwrite(dati,strings,obuff,1)!=1)//se ci sono problemi di I/O si ferma
   {
     return 2;
   }
   i=0,err=0;
   do
   {
      for(stat=0,c=0;(stat<2&&ctrl==1)||(stat<1&&ctrl!=1);c++)	//attesa dato
      {
	usleep(10);
	stat=serread(dati,ibuff); 	//legge dato
	if(stat==1&&ctrl==1&&err==0)	//Deve leggere due bytes (tranne che nell'ultima trasmissione), ma 
	{				// la sfiga capita spesso. Se becca il momento in cui c'e' un solo
	  err=1;			// byte, lo recupera e sistema le cose
	  itemp=ibuff[0];
	}
	else if(stat==1&&ctrl==1&&err==1)
	{
	  err=0;
	  ibuff[1]=ibuff[0];
	  ibuff[0]=itemp;
	  c=0;
	  stat=2;
	}
	if(c==5000)			//50000*10ns = 0,5 sec di attesa dato
	{
	  close(dati->serPort);		//chiude porta seriale
	  serioerr(dati,strings);	//timeout
	  return 2;
	}
      }
      ch=ibuff[0]&0x000000FF;		//pulisce il dato trasformando in byte
      cj=ibuff[1]&0x000000FF;
      switch(ctrl)
      {
	case 0:				//handshaking iniziale
	{
	  if(ch==ser_ackwrite&&(dati->eesize)<257)	//comando ricevuto, invia primo blocco
	  {
	    if(DEBUG)			//visualizza output di debug
	    {
	      if(dati->cmdline)
	      {
		printf("ACK_WRITE <- MCU\n");
	      }
	      else
	      {
		printw("ACK_WRITE <- MCU\n");
		refresh();
	      }
	    }
	    i=0;
	    ctrl=1;
	    obuff[0]=n;
	    obuff[1]=dati->ee[(n*m)+n];
	    if(DEBUG)			//visualizza output di debug
	    {
	      if(dati->cmdline)
	      {
		printf("PC -> ADDR (%i)\n",obuff[0]);
		printf("PC -> DATA (%i)\n",obuff[1]);
	      }
	      else
	      {
		printw("PC -> ADDR (%i)\n",obuff[0]);
		printw("PC -> DATA (%i)\n",obuff[1]);
		refresh();
	      }
	    }
	    if(serwrite(dati,strings,obuff,2)!=2)	//se ci sono problemi di I/O si ferma
	    {
	      close(dati->serPort);	//chiude porta seriale
	      serioerr(dati,strings);	//visualizza errore
	      return 2;
	    }
	  }
	  else if(ch==ser_ackwrext&&(dati->eesize)>256)	//comando ricevuto, invia MSB indirizzo esteso
	  {
	    i=0;
	    ctrl=6;
	    obuff[0]=m;
	    if(DEBUG)			//visualizza output di debug
	    {
	      if(dati->cmdline)
	      {
		printf("PC -> MSB ADDR (%i)\n",obuff[0]);
	      }
	      else
	      {
		printw("PC -> MSB ADDR (%i)\n",obuff[0]);
		refresh();
	      }
	    }
	    if(serwrite(dati,strings,obuff,1)!=1)	//se ci sono problemi di I/O si ferma
	    {
	      close(dati->serPort);	//chiude porta seriale
	      serioerr(dati,strings);	//visualizza errore
	      return 2;
	    }
	  }
	  else if(ch==ser_ackrcver)	//errore ricezione, riparte
	  {
	    usleep(10000);
	    ctrl=0;
	    if((dati->eesize)>256)
	    {
	      obuff[0]=ser_cmdwrext;	//scrittura con due bytes di indirizzo
	      if(DEBUG)			//visualizza output di debug
	      {
		if(dati->cmdline)
		{
		  printf("ACK_RCVER <- MCU\n");
		  printf("PC -> CMD_WREXT\n");
		}
		else
		{
		  printw("ACK_RCVER <- MCU\n");
		  printw("PC -> CMD_WREXT\n");
		  refresh();
		}
	      }
	    }
	    else
	    {
	      obuff[0]=ser_cmdwrite;	//scrittura con un byte di indirizzo
	      if(DEBUG)			//visualizza output di debug
	      {
		if(dati->cmdline)
		{
		  printf("ACK_RCVER <- MCU\n");
		  printf("PC -> CMD_WRITE\n");
		}
		else
		{
		  printw("ACK_RCVER <- MCU\n");
		  printw("PC -> CMD_WRITE\n");
		  refresh();
		}
	      }
	    }
	    if(serwrite(dati,strings,obuff,1)!=1)	//se ci sono problemi di I/O si ferma
	    {
	      close(dati->serPort);	//chiude porta seriale
	      serioerr(dati,strings);	//visualizza errore
	      return 2;
	    }
	    i++;
	  }
	  else if(ch==ser_acknowrt&&(dati->eesize)<257)
	  {
	    close(dati->serPort);	//chiude porta seriale
	    if(DEBUG)			//visualizza output di debug
	    {
	      if(dati->cmdline)
	      {
		printf("ACK_NOWRT <- MCU\n");
	      }
	      else
	      {
		printw("ACK_NOWRT <- MCU\n");
		refresh();
	      }
	    }
	    if(dati->cmdline)		//visualizza messaggio di errore in modalita' parametrica
	    {
	      printf("%s %i bytes, %s\n",strings->eesize001,dati->eesize,strings->eesize006);//EEPROM size is set to * bytes, but the MCU reported more.
	    }
	    else			//visualizza messaggio di errore in modalita' grafica
	    {
	      eediffscreen2(dati,strings);
	    }
	    return 3;
	  }
	  else if(ch==ser_acknoext&&(dati->eesize)>256)
	  {
	    close(dati->serPort);	//chiude porta seriale
	    if(DEBUG)			//visualizza output di debug
	    {
	      if(dati->cmdline)
	      {
		printf("ACK_NOEXT <- MCU\n");
	      }
	      else
	      {
		printw("ACK_NOEXT <- MCU\n");
		refresh();
	      }
	    }
	    if(dati->cmdline)		//visualizza messaggio di errore in modalita' parametrica
	    {
	      printf("%s %i bytes, %s\n",strings->eesize001,dati->eesize,strings->eesize006);//EEPROM size is set to * bytes, but the MCU reported less.
	    }
	    else			//visualizza messaggio di errore in modalita' grafica
	    {
	      eediffscreen1(dati,strings);
	    }
	    return 3;
	  }
	  else				//che cavolo ha letto?
	  {
	    i++;
	  }
	  break;
	}
	case 1:				//ritorna indirizzo e dato per controllo
	{
	  if(ch==n&&cj==dati->ee[(n*m)+n])	//dato e indirizzo corretti
	  {
	    i=0;
	    ctrl=5;
	    obuff[0]=ser_ackindok;
	    obuff[1]=ser_ackdtok;
	    if(DEBUG)			//visualizza output di debug
	    {
	      if(dati->cmdline)
	      {
		printf("ADDR (%i) <- MCU\n",ch);
		printf("DATA (%i) <- MCU\n",cj);
		printf("PC -> ACK_INDOK\n");
		printf("PC -> ACK_DTOK\n");
	      }
	      else
	      {
		printw("ADDR (%i) <- MCU\n",ch);
		printw("DATA (%i) <- MCU\n",cj);
		printw("PC -> ACK_INDOK\n");
		printw("PC -> ACK_DTOK\n");
		refresh();
	      }
	    }
	    if(serwrite(dati,strings,obuff,2)!=2)	//se ci sono problemi di I/O si ferma
	    {
	      close(dati->serPort);	//chiude porta seriale
	      serioerr(dati,strings);	//visualizza errore
	      return 2;
	    }
	  }
	  else if(ch==n&&cj!=dati->ee[(n*m)+n])	//dato non corretto
	  {
	    ctrl=2;
	    obuff[0]=ser_ackdter;	//invia ack
	    obuff[1]=dati->ee[(n*m)+n];
	    if(DEBUG)			//visualizza output di debug
	    {
	      if(dati->cmdline)
	      {
		printf("ADDR (%i) <- MCU\n",ch);
		printf("DATA (%i) <- MCU\n",cj);
		printf("PC -> ACK_DTER\n");
		printf("PC -> DATA (%i)\n",obuff[1]);
	      }
	      else
	      {
		printw("ADDR (%i) <- MCU\n",ch);
		printw("DATA (%i) <- MCU\n",cj);
		printw("PC -> ACK_DTER\n");
		printw("PC -> DATA (%i)\n",obuff[1]);
		refresh();
	      }
	    }
	    if(serwrite(dati,strings,obuff,2)!=2)	//se ci sono problemi di I/O si ferma
	    {
	      close(dati->serPort);	//chiude porta seriale
	      serioerr(dati,strings);	//visualizza errore
	      return 2;
	    }
	    i++;
	  }
	  else if(ch!=n&&cj==dati->ee[(n*m)+n])	//indirizzo non corretto
	  {
	    ctrl=3;
	    obuff[0]=ser_ackinder;	//invia ack
	    obuff[1]=n;
	    if(DEBUG)			//visualizza output di debug
	    {
	      if(dati->cmdline)
	      {
		printf("ADDR (%i) <- MCU\n",ch);
		printf("DATA (%i) <- MCU\n",cj);
		printf("PC -> ACK_INDER\n");
		printf("PC -> ADDR (%i)\n",obuff[1]);
	      }
	      else
	      {
		printw("ADDR (%i) <- MCU\n",ch);
		printw("DATA (%i) <- MCU\n",cj);
		printw("PC -> ACK_INDER\n");
		printw("PC -> ADDR (%i)\n",obuff[1]);
		refresh();
	      }
	    }
	    if(serwrite(dati,strings,obuff,2)!=2)	//se ci sono problemi di I/O si ferma
	    {
	      close(dati->serPort);	//chiude porta seriale
	      serioerr(dati,strings);	//visualizza errore
	      return 2;
	    }
	    i++;
	  }
	  else if(ch==ser_ackrcver)	//errore generale, riparte
	  {
	    usleep(10000);
	    ctrl=0;
	    if((dati->eesize)>256)
	    {
	      obuff[0]=ser_cmdwrext;	//scrittura con due bytes di indirizzo
	      if(DEBUG)			//visualizza output di debug
	      {
		if(dati->cmdline)
		{
		  printf("ACK_RCVER <- MCU\n");
		  printf("PC -> CMD_WREXT\n");
		}
		else
		{
		  printw("ACK_RCVER <- MCU\n");
		  printw("PC -> CMD_WREXT\n");
		  refresh();
		}
	      }
	    }
	    else
	    {
	      obuff[0]=ser_cmdwrite;	//scrittura con un byte di indirizzo
	      if(DEBUG)			//visualizza output di debug
	      {
		if(dati->cmdline)
		{
		  printf("ACK_RCVER <- MCU\n");
		  printf("PC -> CMD_WRITE\n");
		}
		else
		{
		  printw("ACK_RCVER <- MCU\n");
		  printw("PC -> CMD_WRITE\n");
		  refresh();
		}
	      }
	    }
	    if(serwrite(dati,strings,obuff,1)!=1)	//se ci sono problemi di I/O si ferma
	    {
	      close(dati->serPort);	//chiude porta seriale
	      serioerr(dati,strings);	//visualizza errore
	      return 2;
	    }
	    i++;
	  }
	  else				//dato e indirizzo non corretti
	  {
	    ctrl=4;
	    obuff[0]=ser_ackblker;
	    if(DEBUG)			//visualizza output di debug
	    {
	      if(dati->cmdline)
	      {
		printf("PC -> ACK_BLKER\n");
	      }
	      else
	      {
		printw("PC -> ACK_BLKER\n");
		refresh();
	      }
	    }
	    if(serwrite(dati,strings,obuff,1)!=1)	//se ci sono problemi di I/O si ferma
	    {
	      close(dati->serPort);	//chiude porta seriale
	      serioerr(dati,strings);	//visualizza errore
	      return 2;
	    }
	    i++;
	  }
	  break;
	}
	case 2:				//ricezione nuovo dato per controllo
	{
	  if(ch==dati->ee[(n*m)+n])
	  {
	    ctrl=5;
	    i=0;
	    obuff[0]=ser_ackdtok;
	    if(DEBUG)			//visualizza output di debug
	    {
	      if(dati->cmdline)
	      {
		printf("DATA (%i) <- MCU\n",ch);
		printf("PC -> ACK_DTOK\n");
	      }
	      else
	      {
		printw("DATA (%i) <- MCU\n",ch);
		printw("PC -> ACK_DTOK\n");
		refresh();
	      }
	    }
	    if(serwrite(dati,strings,obuff,1)!=1)	//se ci sono problemi di I/O si ferma
	    {
	      close(dati->serPort);	//chiude porta seriale
	      serioerr(dati,strings);	//visualizza errore
	      return 2; 
	    }
	  }
	  else if(ch==ser_ackrcver)	//errore generale, riparte
	  {
	    usleep(10000);
	    ctrl=0;
	    if((dati->eesize)>256)
	    {
	      obuff[0]=ser_cmdwrext;	//scrittura con due bytes di indirizzo
	      if(DEBUG)			//visualizza output di debug
	      {
		if(dati->cmdline)
		{
		  printf("ACK_RCVER <- MCU\n");
		  printf("PC -> CMD_WREXT\n");
		}
		else
		{
		  printw("ACK_RCVER <- MCU\n");
		  printw("PC -> CMD_WREXT\n");
		  refresh();
		}
	      }
	    }
	    else
	    {
	      obuff[0]=ser_cmdwrite;	//scrittura con un byte di indirizzo
	      if(DEBUG)			//visualizza output di debug
	      {
		if(dati->cmdline)
		{
		  printf("ACK_RCVER <- MCU\n");
		  printf("PC -> CMD_WRITE\n");
		}
		else
		{
		  printw("ACK_RCVER <- MCU\n");
		  printw("PC -> CMD_WRITE\n");
		  refresh();
		}
	      }
	    }
	    if(serwrite(dati,strings,obuff,1)!=1)	//se ci sono problemi di I/O si ferma
	    {
	      close(dati->serPort);	//chiude porta seriale
	      serioerr(dati,strings);	//visualizza errore
	      return 2;
	    }
	    i++;
	  }
	  else				//dato errato
	  {
	    obuff[0]=ser_ackdter;	//invia ack
	    obuff[1]=dati->ee[(n*m)+n];
	    if(DEBUG)			//visualizza output di debug
	    {
	      if(dati->cmdline)
	      {
		printf("DATA (%i) <- MCU\n",ch);
		printf("PC -> ACK_DTER\n");
		printf("PC -> DATA (%i)\n",obuff[1]);
	      }
	      else
	      {
		printw("DATA (%i) <- MCU\n",ch);
		printw("PC -> ACK_DTER\n");
		printw("PC -> DATA (%i)\n",obuff[1]);
		refresh();
	      }
	    }
	    if(serwrite(dati,strings,obuff,2)!=2)	//se ci sono problemi di I/O si ferma
	    {
	      close(dati->serPort);	//chiude porta seriale
	      serioerr(dati,strings);	//visualizza errore
	      return 2;
	    }
	    i++;
	  }
	  break;
	}
	case 3:				//ricezione nuovo indirizzo per controllo
	{
	  if(ch==n)
	  {
	    ctrl=5;
	    i=0;
	    obuff[0]=ser_ackindok;
	    if(DEBUG)			//visualizza output di debug
	    {
	      if(dati->cmdline)
	      {
		printf("ADDR (%i) <- MCU\n",ch);
		printf("PC -> ACK_INDOK\n");
	      }
	      else
	      {
		printw("ADDR (%i) <- MCU\n",ch);
		printw("PC -> ACK_INDOK\n");
		refresh();
	      }
	    }
	    if(serwrite(dati,strings,obuff,1)!=1)	//se ci sono problemi di I/O si ferma
	    {
	      close(dati->serPort);	//chiude porta seriale
	      serioerr(dati,strings);	//visualizza errore
	      return 2;
	    }
	  }
	  else if(ch==ser_ackrcver)	//errore generale, riparto
	  {
	    usleep(10000);
	    ctrl=0;
	    if((dati->eesize)>256)
	    {
	      obuff[0]=ser_cmdwrext;	//scrittura con due bytes di indirizzo
	      if(DEBUG)			//visualizza output di debug
	      {
		if(dati->cmdline)
		{
		  printf("ACK_RCVER <- MCU\n");
		  printf("PC -> CMD_WREXT\n");
		}
		else
		{
		  printw("ACK_RCVER <- MCU\n");
		  printw("PC -> CMD_WREXT\n");
		  refresh();
		}
	      }
	    }
	    else
	    {
	      obuff[0]=ser_cmdwrite;	//scrittura con un byte di indirizzo
	      if(DEBUG)			//visualizza output di debug
	      {
		if(dati->cmdline)
		{
		  printw("ACK_RCVER <- MCU\n");
		  printf("PC -> CMD_WRITE\n");
		}
		else
		{
		  printw("ACK_RCVER <- MCU\n");
		  printw("PC -> CMD_WRITE\n");
		  refresh();
		}
	      }
	    }
	    if(serwrite(dati,strings,obuff,1)!=1)	//se ci sono problemi di I/O si ferma
	    {
	      close(dati->serPort);	//chiude porta seriale
	      serioerr(dati,strings);	//visualizza errore
	      return 2;
	    }
	    i++;
	  }
	  else				//indirizzo errato
	  {
	    obuff[0]=ser_ackinder;	//invia ack
	    obuff[1]=n;
	    if(DEBUG)			//visualizza output di debug
	    {
	      if(dati->cmdline)
	      {
		printf("ADDR (%i) <- MCU\n",ch);
		printf("PC -> ACK_INDER\n");
		printf("PC -> ADDR (%i)\n",obuff[1]);
	      }
	      else
	      {
		printw("ADDR (%i) <- MCU\n",ch);
		printw("PC -> ACK_INDER\n");
		printw("PC -> ADDR (%i)\n",obuff[1]);
		refresh();
	      }
	    }
	    if(serwrite(dati,strings,obuff,2)!=2)	//se ci sono problemi di I/O si ferma
	    {
	      close(dati->serPort);	//chiude porta seriale
	      serioerr(dati,strings);	//visualizza errore
	      return 2;
	    }
	    i++;
	  }
	  break;
	}
	case 4:				//ricezione ack errore blocco
	{
	  if(ch==ser_ackblker)		//reinvia dato e indirizzo
	  {
	    ctrl=1;
	    i=0;
	    obuff[0]=n;
	    obuff[1]=dati->ee[(n*m)+n];
	    if(DEBUG)			//visualizza output di debug
	    {
	      if(dati->cmdline)
	      {
		printf("ACK_BLKER <- MCU\n");
		printf("PC -> ADDR (%i)\n",obuff[0]);
		printf("PC -> DATA (%i)\n",obuff[1]);
	      }
	      else
	      {
		printw("ACK_BLKER <- MCU\n");
		printw("PC -> ADDR (%i)\n",obuff[0]);
		printw("PC -> DATA (%i)\n",obuff[1]);
		refresh();
	      }
	    }
	    if(serwrite(dati,strings,obuff,2)!=2)	//se ci sono problemi di I/O si ferma
	    {
	      close(dati->serPort);	//chiude porta seriale
	      serioerr(dati,strings);	//visualizza errore
	      return 2;
	    }
	  }
	  else				//errore generale, riparte
	  {
	    usleep(10000);
	    ctrl=0;
	    if((dati->eesize)>256)
	    {
	      obuff[0]=ser_cmdwrext;	//scrittura con due bytes di indirizzo
	      if(DEBUG)			//visualizza output di debug
	      {
		if(dati->cmdline)
		{
		  printf("??? <- MCU\n");
		  printf("PC -> CMD_WREXT\n");
		}
		else
		{
		  printw("??? <- MCU\n");
		  printw("PC -> CMD_WREXT\n");
		  refresh();
		}
	      }
	    }
	    else
	    {
	      obuff[0]=ser_cmdwrite;	//scrittura con un byte di indirizzo
	      if(DEBUG)			//visualizza output di debug
	      {
		if(dati->cmdline)
		{
		  printf("??? <- MCU\n");
		  printf("PC -> CMD_WRITE\n");
		}
		else
		{
		  printw("??? <- MCU\n");
		  printw("PC -> CMD_WRITE\n");
		  refresh();
		}
	      }
	    }
	    if(serwrite(dati,strings,obuff,1)!=1)	//se ci sono problemi di I/O si ferma
	    {
	      close(dati->serPort);	//chiude porta seriale
	      serioerr(dati,strings);	//visualizza errore
	      return 2;
	    }
	    i++;
	  }
	  break;
	}
	case 5:				//ricezione scrittura EEPROM ok
	{
	  if(ch==ser_ackwrtok)		//scrittura EEPROM OK
	  {
	    if(DEBUG)			//visualizza output di debug
	    {
	      if(dati->cmdline)
	      {
		printf("ACK_WRTOK <- MCU\n");
	      }
	      else
	      {
		printw("ACK_WRTOK <- MCU\n");
		refresh();
	      }
	    }
	    dati->eemod[n]=false;	//clear flag di modifica
	    do				//se modalita' scrittura modifica, cerca dato. Altrimenti carica il successivo
	    {
	      n++;			//carica prossimo indirizzo modificato
	      if(n==256)
	      {
		m++;			//incrementa MSB indirizzo esteso
		n=0;
	      }
	    }
	    while(!(dati->eemod[(n*m)+n])&&(256*m)+n<(dati->eesize)-1&&(dati->eemode)=='m');
	    ctrl=0;
	    i=0; 			//reset contatore timeout
	    for (z=0;z<65;z++)
	    {
	      if(z<(((256*m)+n)/((dati->eesize)/64)))
		barra[z]=true;
	      else
		barra[z]=false;
	    }
	    barra[(int)(n/((dati->eesize)/64))]=true;	//aggiorna posizione barra
	    if(dati->cmdline)		//visualizza avanzamento testuale
	    {
	      printf("\tbyte %i %s %i\n",(n+1),strings->str003,dati->eesize);//byte * of *
	    }
	    else			//visualizza barra grafica
	    {
	      graphbarmove(dati,strings,barra,n);
	    }
	    if((256*m)+n==((dati->eesize)-1))	//fine comunicazione
	    {	   
	      close(dati->serPort);	//chiude porta seriale
	      if(dati->cmdline)		//visualizza avanzamento testuale
	      {
		printf("\tbyte %i %s %i\n",(n+1),strings->str003,dati->eesize);//byte * of *
	      }
	      else			//visualizza barra grafica
	      {
// 		mvprintw(12,2,"%s",strings->strike001);	//Strike a key to continue...
// 		curs_set(0);
// 		refresh();
		getch();
	      }
	      return 0; 		//fine comunicazione
	    }
	    else			//prossimo byte
	    {
	      if((dati->eesize)>256)
	      {
		obuff[0]=ser_cmdwrext;	//scrittura con due bytes di indirizzo
		if(DEBUG)		//visualizza output di debug
		{
		  if(dati->cmdline)
		  {
		    printf("PC -> CMD_WREXT\n");
		  }
		  else
		  {
		    printw("PC -> CMD_WREXT\n");
		    refresh();
		  }
		}
	      }
	      else
	      {
		obuff[0]=ser_cmdwrite;	//scrittura con un byte di indirizzo
		if(DEBUG)		//visualizza output di debug
		{
		  if(dati->cmdline)
		  {
		    printf("PC -> CMD_WRITE\n");
		  }
		  else
		  {
		    printw("PC -> CMD_WRITE\n");
		    refresh();
		  }
		}
	      }
	      if(serwrite(dati,strings,obuff,1)!=1)	//se ci sono problemi di I/O si ferma
	      {
		close(dati->serPort);	//chiude porta seriale
		serioerr(dati,strings);	//visualizza errore
		return 2;
	      }
	    }
	  }
	  else if(ch==ser_ackwrter)	//scrittura EEPROM ERR
	  {
	    close(dati->serPort);	//chiude porta seriale
	    if(DEBUG)			//visualizza output di debug
	    {
	      if(dati->cmdline)
	      {
		printf("ACK_WRTER <- MCU\n");
	      }
	      else
	      {
		printw("ACK_WRTER <- MCU\n");
		refresh();
	      }
	    }
	    if(dati->cmdline)		//visualizza messaggio di errore in modalita' parametrica
	    {
	      printf("%s %i.\n",strings->err005,(n+1));//System board reported an error writing on EEPROM cell *.
	      printf("%s\n\n",strings->err006);//Write process will be terminated.
	    }
	    else			//visualizza messaggio di errore in modalita' grafica
	    {
	      eediffscreen4(dati,strings,n);
	    }
	    return 5; 			//fine comunicazione
	  }
	  else				//errore di ricezione, riparte
	  {
	    ctrl=0;
	    usleep(10000);
	    if((dati->eesize)>256)
	    {
	      obuff[0]=ser_cmdwrext;	//scrittura con due bytes di indirizzo
	      if(DEBUG)			//visualizza output di debug
	      {
		if(dati->cmdline)
		{
		  printf("??? <- MCU\n");
		  printf("PC -> CMD_WREXT\n");
		}
		else
		{
		  printw("??? <- MCU\n");
		  printw("PC -> CMD_WREXT\n");
		  refresh();
		}
	      }
	    }
	    else
	    {
	      obuff[0]=ser_cmdwrite;	//scrittura con un byte di indirizzo
	      if(DEBUG)			//visualizza output di debug
	      {
		if(dati->cmdline)
		{
		  printf("??? <- MCU\n");
		  printf("PC -> CMD_WRITE\n");
		}
		else
		{
		  printw("??? <- MCU\n");
		  printw("PC -> CMD_WRITE\n");
		  refresh();
		}
	      }
	    }
	    if(serwrite(dati,strings,obuff,1)!=1)	//se ci sono problemi di I/O si ferma
	    {
	      close(dati->serPort);	//chiude porta seriale
	      serioerr(dati,strings);	//visualizza errore
	      return 2;
	    }
	    i++;
	  }
	  break;
	}
	case 6:				//ricezione MSB indirizzo per controllo
	{
	  if(ch==m)			//MSB indirizzo OK, invio LSB indirizzo esteso e dato
	  {
	    obuff[0]=ser_ackindok;
	    obuff[1]=m;
	    if(DEBUG)			//visualizza output di debug
	    {
	      if(dati->cmdline)
	      {
		printf("MSB ADDR (%i) <- MCU\n",ch);
		printf("PC -> ACK_INDOK\n");
		printf("PC -> MSB ADDR (%i)\n",obuff[1]);
	      }
	      else
	      {
		printw("MSB ADDR (%i) <- MCU\n",ch);
		printw("PC -> ACK_INDOK\n");
		printw("PC -> MSB ADDR (%i)\n",obuff[1]);
		refresh();
	      }
	    }
	    if(serwrite(dati,strings,obuff,2)!=2)	//se ci sono problemi di I/O si ferma
	    {
	      close(dati->serPort);	//chiude porta seriale
	      serioerr(dati,strings);	//visualizza errore
	      return 2;
	    }
	    usleep(6000);		//attende il micro
	    ctrl=1;
	    i=0;
	    obuff[0]=n;			//invia LSB e dato
	    obuff[1]=dati->ee[(n*m)+n];
	    if(DEBUG)			//visualizza output di debug
	    {
	      if(dati->cmdline)
	      {
		printf("...wait...\n");
		printf("PC -> ADDR (%i)\n",obuff[0]);
		printf("PC -> DATA (%i)\n",obuff[1]);
	      }
	      else
	      {
		printw("...wait...\n");
		printw("PC -> ADDR (%i)\n",obuff[0]);
		printw("PC -> DATA (%i)\n",obuff[1]);
		refresh();
	      }
	    }
	    if(serwrite(dati,strings,obuff,2)!=2)	//se ci sono problemi di I/O si ferma
	    {
	      close(dati->serPort);	//chiude porta seriale
	      serioerr(dati,strings);	//visualizza errore
	      return 2;
	    }
	  }
	  else if(ch==ser_ackrcver)	//riparte
	  {
	    usleep(10000);
	    ctrl=0;
	    if((dati->eesize)>256)
	    {
	      obuff[0]=ser_cmdwrext;	//scrittura con due bytes di indirizzo
	      if(DEBUG)			//visualizza output di debug
	      {
		if(dati->cmdline)
		{
		  printf("ACK_RCVER <- MCU\n");
		  printf("PC -> CMD_WREXT\n");
		}
		else
		{
		  printw("ACK_RCVER <- MCU\n");
		  printw("PC -> CMD_WREXT\n");
		  refresh();
		}
	      }
	    }
	    else
	    {
	      obuff[0]=ser_cmdwrite;	//scrittura con un byte di indirizzo
	      if(DEBUG)			//visualizza output di debug
	      {
		if(dati->cmdline)
		{
		  printf("ACK_RCVER <- MCU\n");
		  printf("PC -> CMD_WRITE\n");
		}
		else
		{
		  printw("ACK_RCVER <- MCU\n");
		  printw("PC -> CMD_WRITE\n");
		  refresh();
		}
	      }
	    }
	    if(serwrite(dati,strings,obuff,1)!=1)	//se ci sono problemi di I/O si ferma
	    {
	      close(dati->serPort);	//chiude porta seriale
	      serioerr(dati,strings);	//visualizza errore
	      return 2;
	    }
	    i++;
	  }
	  else				//MSB indirizzo sbagliato, reinvia
	  {
	    obuff[0]=ser_ackinder;
	    obuff[1]=m;
	    if(DEBUG)			//visualizza output di debug
	    {
	      if(dati->cmdline)
	      {
		printf("MSB ADDR (%i) <- MCU\n",ch);
		printf("PC -> ACK_INDER\n");
		printf("PC -> MSB_ADDR (%i)\n",obuff[1]);
	      }
	      else
	      {
		printw("MSB ADDR (%i) <- MCU\n",ch);
		printw("PC -> ACK_INDER\n");
		printw("PC -> MSB_ADDR (%i)\n",obuff[1]);
		refresh();
	      }
	    }
	    if(serwrite(dati,strings,obuff,2)!=2)	//se ci sono problemi di I/O si ferma
	    {
	      close(dati->serPort);	//chiude porta seriale
	      serioerr(dati,strings);	//visualizza errore
	      return 2; 
	    }
	    i++;
	  }
	  break;
	}
	default:
	{
	  break;
	}
      }
   }	
   while(i<10);
   close(dati->serPort);		//chiude porta seriale
   serioerr(dati,strings);		//se arriva qui ci sono stati troppi errori di comunicazione
   return 2;
}


/********************************
 *          serwrite()          *
 ********************************
 Invia bytes alla porta

Variabili esterne:
dati=struttura SYSDATA (vedere params.h)
chars=numero caratteri da inviare
dato[]=buffer dato da inviare

Variabili interne:
n=bytes scritti dal comando scrittura
*/
int serwrite(SYSDATA *dati,SYSTEXT *strings,char *dato,int chars)
{
   int n;
   n=write(dati->serPort, dato, chars);
   if (n < 0)				//errore?
   {
      serioerr(dati,strings);
      return 0;				//0 caratteri inviati
   }
   return n;				//ritorna il numero di caratteri inviati
}


/********************************
 *          serread()           *
 ********************************
 Legge dalla porta

Variabili esterne:
dati=struttura SYSDATA (vedere params.h)
dato[]=buffer dato da ricevere

Variabili interne:
c=bytes letti dal comando lettura
*/
int serread(SYSDATA *dati,char *dato)
{
   int c;
   fcntl(dati->serPort, F_SETFL, FNDELAY);	//imposta porta per non attendere dati
   c=read(dati->serPort, dato, 254);
   return c;				//ritorna il numero di caratteri ricevuti
}


/********************************
 *          serenum()           *
 ********************************
 Enumera le porte seriali

Variabili esterne:
dati=struttura SYSDATA (vedere params.h)

Variabili interne:
i=contatore generico
m=contatore numero progressivo porta
n=contatore posizione vettore porte
*/
int serenum(SYSDATA *dati)
{
   int m=0,n=0,i;
   dati->port[0]='/';			//carica indirizzo porta
   dati->port[1]='d';
   dati->port[2]='e';
   dati->port[3]='v';
   dati->port[4]='/';
   dati->port[5]='t';
   dati->port[6]='t';
   dati->port[7]='y';
   dati->port[8]='S';
   do					//cerca le prime 32 porte /dev/ttyS*
   {
     dati->port[9]=m+48;
     dati->port[10]='\0';
     if(fileexists(dati->port))
     {
       for(i=0;i<15;i++)
	    dati->ports[(n*maxport)+i]=dati->port[i];
       n++;
     }
     m++;
   }
   while(m<32);
   dati->port[8]='U';			//modifica indirizzo porta
   dati->port[9]='S';
   dati->port[10]='B';
   m=0;
   do					//cerca le prime 32 porte /dev/ttyUSB*
   {
     dati->port[11]=m+48;
     dati->port[12]='\0';
     if(fileexists(dati->port))
     {
       for(i=0;i<15;i++)
	 dati->ports[(n*maxport)+i]=dati->port[i];
       n++;
     }
     m++;
   }
   while(m<32);
   return n;
}


/********************************
 *          initport()          *
 ********************************
 Inizializza la porta
 
Variabili esterne:
dati=struttura SYSDATA (vedere params.h)

Variabili interne:
options=struttura parametri pporta seriale
*/
int initport(SYSDATA *dati)
{
    struct termios options;
    tcgetattr(dati->serPort, &options);	//carica settaggi attuali porta
    switch(dati->speed)			//imposta il baud rate
    {
       case 300:
       {
	  cfsetispeed(&options, B300);
	  cfsetospeed(&options, B300);
	  break;
       }
       case 600:
       {
	  cfsetispeed(&options, B600);
	  cfsetospeed(&options, B600);
	  break;
       }
       case 1200:
       {
	  cfsetispeed(&options, B1200);
	  cfsetospeed(&options, B1200);
	  break;
       }
       case 2400:
       {
	  cfsetispeed(&options, B2400);
	  cfsetospeed(&options, B2400);
	  break;
       }
       case 4800:
       {
	  cfsetispeed(&options, B4800);
	  cfsetospeed(&options, B4800);
	  break;
       }
       case 9600:
       {
	  cfsetispeed(&options, B9600);
	  cfsetospeed(&options, B9600);
	  break;
       }
       case 19200:
       {
	  cfsetispeed(&options, B19200);
	  cfsetospeed(&options, B19200);
	  break;
       }
       case 38400:
       {
	  cfsetispeed(&options, B38400);
	  cfsetospeed(&options, B38400);
	  break;
       }
       case 57600:
       {
	  cfsetispeed(&options, B57600);
	  cfsetospeed(&options, B57600);
	  break;
       }
       case 115200:
       {
	  cfsetispeed(&options, B115200);
	  cfsetospeed(&options, B115200);
	  break;
       }
    }
    options.c_cflag |= (CLOCAL | CREAD);//abilita ricezione, set local mode
    options.c_cflag &= ~PARENB;		//trasmissione 8N1
    options.c_cflag &= ~CSTOPB;
    options.c_cflag &= ~CSIZE;
    options.c_cflag |= CS8;
    options.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG);  //setta raw input
    options.c_iflag &= ~(IXON | IXOFF | IXANY | IGNCR | ICRNL);	//disabilita flow control e CR
    options.c_oflag &= ~OPOST;		//setta raw output
    tcsetattr(dati->serPort, TCSAFLUSH, &options);	// Applica le impostazioni
    return 1;
}


/********************************
 *            ping()           *
 ********************************
 Test connessione seriale

Variabili esterne:
dati=struttura SYSDATA (vedere params.h)

Variabili interne:
c=contatore timeout lettura
count=contatore ripetizioni ping
ibuff[]=buffer lettura seriale
obuff[]=buffer scrittura seriale
stat=bytes letti dalla porta

ritorno:
0=tutto ok
1=errore apertura porta seriale
2=errore I/O porta seriale
*/
int ping(SYSDATA *dati,SYSTEXT *strings)
{
  char obuff[1];
  char *ibuff=(char*)calloc(sizeof(char),254);
  int stat,c,count=0;
  obuff[0]=ser_cmdping;
  dati->serPort = open(dati->port, O_RDWR | O_NOCTTY | O_NDELAY);	//prova ad aprire porta
  if (dati->serPort == -1)
  {
    seropenerr(dati,strings);		//errore apertura porta
    free(ibuff);
    return 1;
  }
  else
  {
    fcntl(dati->serPort, F_SETFL, FNDELAY);	//configura porta in ricezione per ritornare subito (vedere guida posix)
    initport(dati);			//set velocita' comunicazione
  }
  if(DEBUG)				//visualizza output di debug
  {
    if(dati->cmdline)
    {
      printf("PC -> CMD_PING\n");
    }
    else
    {
      printw("PC -> CMD_PING\n");
      refresh();
    }
  }
  if(serwrite(dati,strings,obuff,1)==0)	//se ci sono problemi di I/O si ferma
  {
    close(dati->serPort);		//chiude porta seriale
    serioerr(dati,strings);		//visualizza errore
    free(ibuff);
    return 2;
  }
  do
  {
    for(stat=0,c=0;stat<1;c++)		//attesa dato, non piu' di 500ms
    {
      usleep(10000);
      stat=serread(dati,ibuff);		//legge dato
      if(c==500)			//timeout?
      {
	close(dati->serPort);		//chiude porta seriale
	serioerr(dati,strings);		//visualizza errore
	free(ibuff);
	return 2;
      }
    }
    if((ibuff[0]&0x000000FF)==ser_ackping)
    {
      close(dati->serPort);		//chiude porta seriale
      if(DEBUG)				//visualizza output di debug
      {
	if(dati->cmdline)
	{
	  printf("ACK_PING <- MCU\n");
	}
	else
	{
	  printw("ACK_PING <- MCU\n");
	  refresh();
	}
      }
      free(ibuff);
      if(dati->cmdline)			//visualizza messaggio testuale
      {
	printf("%s\n\n",strings->success006);//Ping test executed successfully.
      }
      else				//visualizza messaggio grafico
      {
	cnfscreen3(dati,strings);
      }
      return 0;
    }
    else
    {
      if(serwrite(dati,strings,obuff,1)==0)	//se ci sono problemi di I/O si ferma
      {
	close(dati->serPort);		//chiude porta seriale
	serioerr(dati,strings);		//visualizza errore
	free(ibuff);
	return 2;
      }
      count++;
    }
  }
  while(count<3);
  close(dati->serPort);			//chiude porta seriale
  serioerr(dati,strings);		//visualizza errore
  free(ibuff);
  return 2;
}
